/*******************************************************************************
 * Copyright (c) 2004, 2007 IBM Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  <a href="mailto:masquill@us.ibm.com>Mike Squillace</a> - initial API and implementation
 *******************************************************************************/

package org.eclipse.actf.accservice.core.win32.msaa;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.eclipse.actf.accservice.AccessibilityServiceException;
import org.eclipse.actf.accservice.AccessibilityServiceManager;
import org.eclipse.actf.accservice.IAccessibilityEventService;
import org.eclipse.actf.accservice.IAccessibilityService;
import org.eclipse.actf.accservice.adapt.win32.msaa.MsaaAccessibilityAdaptor;
import org.eclipse.actf.accservice.core.AccessibleConstants;
import org.eclipse.actf.accservice.core.AccessibleElementTreeWalker;
import org.eclipse.actf.accservice.core.IAccessibleElement;
import org.eclipse.actf.accservice.locate.AccessibleNodeLocator;
import org.eclipse.actf.core.adapt.DefaultAdaptorFactory;
import org.eclipse.actf.model.AbstractGuiModel;
import org.eclipse.actf.model.InvalidComponentException;
import org.eclipse.actf.model.event.IModelEventListener;
import org.eclipse.actf.model.event.ModelEventType;
import org.eclipse.actf.model.locate.INodeLocator;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;


/**
 * a model implementation for a hierarchy of IAccessible components used by the
 * Microsoft Active Accessibility (MSAA) model
 * 
 * @see <a
 *      href="http://msdn2.microsoft.com/en-us/library/ms696165.aspx">IAccessible
 *      interface</a>
 * @author <a href="mailto:masquill@us.ibm.com>Mike Squillace</a>
 * 
 */
public class MsaaGuiModel extends AbstractGuiModel {

	public static final String MSAA_MODEL = "msaa";

	private ModelEventType[] changeEventTypes = null;
	
	/**
	 * create a new MsaaModel
	 */
	public MsaaGuiModel() {
		super(MSAA_MODEL);
		treeNodeWalker = new AccessibleElementTreeWalker();
		setFilters();
	    DefaultAdaptorFactory.getInstance().registerAdaptor(MsaaAccessible.class, new MsaaAccessibilityAdaptor());
	}

	// This method should only be used by classes that extend this class
	protected MsaaGuiModel(String extendedAttrib) {
		super(extendedAttrib);
	}

	/**
	 * {@inheritDoc} returns the accessibleRole of this element if it is of type
	 * <code>MsaaAccessible</code>
	 */
	public String getNodeName(Object element) {
		String accRole = null;
		if (element instanceof MsaaAccessible) {
			try {
				accRole = ((MsaaAccessible) element).getAccessibleRole();
			} catch (InvalidComponentException e) {
			}
		}
		return accRole;
	}

	/**
	 * {@inheritDoc} always returns <code>org.eclipse.actf.accservice.AccessibleElement</code>
	 */
	public String getTypeName (String nodeName) {
		return IAccessibleElement.class.getName();
	}

	protected Class getAccessibleElementType() {
		return MsaaAccessible.class;
	}

	protected Class getAccessibilityEventServiceType() {
		return MsaaAccessibilityEventService.class;
	}

	protected void initEventIdMap() {
		initEventIdMap(getAccessibleElementType(), getAccessibilityEventServiceType(), "EVENT_");
	}

	protected void initEventIdMap(Class accType, Class serviceClass, String fieldPrefix) {
		Field[] fields = serviceClass.getFields();
		for (int f = 0; f < fields.length; ++f) {
			if (fields[f].getName().startsWith(fieldPrefix)) {
				try {
					Object eventId = fields[f].get(null);
					ModelEventType eventType = new ModelEventType(
							fields[f].getName(), eventId, accType);
					eventIdMap.put(eventId, eventType);
				} catch (Exception e) {
					// do nothing - should never happen
				}
			}
		}
	}

	/** {@inheritDoc} */
	public ModelEventType[] getModelEventTypes(Class sourceType) {
		return sourceType == null || sourceType.equals(getAccessibleElementType())
			? (ModelEventType[]) eventIdMap.values().toArray(new ModelEventType[eventIdMap.size()])
			: new ModelEventType[0];
	}

	/** {@inheritDoc}
	 * 
	 * The only optional parameter is an int that specifies flags for MSAA event-processing. If such a flag (or 
	 * combination of flags is not present, <code>DEFAULT_CONTEXT_FLAG</code> will be used.
	 */
	public void registerModelEventListener(IModelEventListener listener, ModelEventType[] eventTypes, Object[] params) {
		registerModelEventListener(listener, eventTypes, params, MsaaAccessibilityService.MSAA_ACCSERVICE_NAME);
	}

	protected void registerModelEventListener(IModelEventListener listener, ModelEventType[] eventTypes,
											  Object[] params, String accServiceName) {
		IAccessibilityService service = null;
		try {
			service = AccessibilityServiceManager.getInstance().getAccessibilityService(accServiceName);
		} catch (AccessibilityServiceException e) {
			LoggingUtil.println(IReporter.SYSTEM_NONFATAL, e);
		}
		
		if (service != null) {
			IAccessibilityEventService eventService = service.getAccessibilityEventService();
			if (eventService != null) {
				List actualIds = new ArrayList();
				for (int i = 0; i < eventTypes.length; ++i) {
					Object eventId = eventTypes[i].getEventId();
					if (eventIdMap.containsKey(eventId)) {
						actualIds.add(eventId);
					}
				}
				eventService.addAccessibilityEventListener(listener, actualIds.toArray(), params);
			}
		}
	}

	/** {@inheritDoc} */
	public void unregisterModelEventListener(IModelEventListener listener, ModelEventType[] eventTypes) {
		unregisterModelEventListener(listener, eventTypes, MsaaAccessibilityService.MSAA_ACCSERVICE_NAME);
	}
	
	protected void unregisterModelEventListener(IModelEventListener listener, ModelEventType[] eventTypes, String accServiceName) {
		IAccessibilityService service = null;
		try {
			service = AccessibilityServiceManager.getInstance().getAccessibilityService(accServiceName);
		} catch (AccessibilityServiceException e) {
			LoggingUtil.println(IReporter.SYSTEM_NONFATAL, e);
		}
		
		if (service != null) {
			IAccessibilityEventService eventService = service.getAccessibilityEventService();
			if (eventService != null) {
				List actualIds = new ArrayList();
				for (int i = 0; i < eventTypes.length; ++i) {
					Object eventId = eventTypes[i].getEventId();
					if (eventIdMap.containsKey(eventId)) {
						actualIds.add(eventId);
					}
				}
				eventService.removeAccessibilityEventListener(listener, actualIds.toArray());
			}
		}
	}

	/** {@inheritDoc} */
	public boolean isVisible(Object comp) {
		boolean result = false;
		try {
			Set state = ((MsaaAccessible) comp).getAccessibleState();
			if (state != null) {
				result = state.contains(AccessibleConstants.STATE_VISIBLE);
			}
		} catch (InvalidComponentException e) {
		}
		return result;
	}

	/** {@inheritDoc} */
	public boolean isValid(Object comp) {
		boolean result = false;
		if (comp != null && comp instanceof MsaaAccessible) {
			try {
				((MsaaAccessible) comp).checkIsValid();
				result = true;
			} catch (InvalidComponentException e) {
				result = false;
			}
		}
		return result;
	}

	/** {@inheritDoc} */
	public boolean requestFocusFor(Object comp) {
		return true;
	}

	/**
	 * will be invoked on the current thread
	 */
	public void asyncInvokeOnUIThread(Runnable runnable) {
		runnable.run();
	}

	/**
	 * will be invoked on the current thread
	 */
	public void invokeOnUIThread(Runnable runnable) {
		runnable.run();
	}

	public boolean isUIThread() {
		return true;
	}

	public boolean isTopDown() {
		return true;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.actf.model.AbstractModel#getNodeLocator()
	 */
	@Override
	public INodeLocator getNodeLocator() {
		if (locator == null){
			locator = new AccessibleNodeLocator();
		}
		return locator;
	}

	public String[] getPackageNames() {
		return new String[0];
	}

}
