/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.accservice.javapp;

import java.awt.Component;
import java.awt.Rectangle;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.accessibility.Accessible;
import javax.accessibility.AccessibleContext;
import javax.accessibility.AccessibleEditableText;
import javax.accessibility.AccessibleIcon;
import javax.accessibility.AccessibleRelation;
import javax.accessibility.AccessibleRelationSet;
import javax.accessibility.AccessibleRole;
import javax.accessibility.AccessibleSelection;
import javax.accessibility.AccessibleState;
import javax.accessibility.AccessibleStateSet;
import javax.accessibility.AccessibleTable;
import javax.accessibility.AccessibleText;
import javax.accessibility.AccessibleValue;
import javax.swing.AbstractButton;
import javax.swing.SwingUtilities;
import javax.swing.text.JTextComponent;

import org.eclipse.actf.accservice.core.AccessibleConstants;
import org.eclipse.actf.accservice.core.IAccessibleEditableTextElement;
import org.eclipse.actf.accservice.core.IAccessibleElement;
import org.eclipse.actf.accservice.core.IAccessibleElement2;
import org.eclipse.actf.accservice.core.IAccessibleImageElement;
import org.eclipse.actf.accservice.core.IAccessibleTableElement;
import org.eclipse.actf.accservice.core.IAccessibleTextElement;
import org.eclipse.actf.model.InvalidComponentException;

/**
 * implementation for the Java Swing Accesibility API's <code>Accessible</code>. All methods are passed onto 
 * the underlying <code>AccessibleContext</code> object.
 * 
 * @see javax.accessibility.Accessible
 * @author Mike Squillace
 *
 */
public class JavaAccessible implements IAccessibleElement2
{

	private static final Map RELATIONKEY_MAP = new HashMap();
	static {
		RELATIONKEY_MAP.put(
			AccessibleRelation.MEMBER_OF,
			AccessibleConstants.RELATION_MEMBER_OF);
		RELATIONKEY_MAP.put(
			AccessibleRelation.LABEL_FOR,
			AccessibleConstants.RELATION_LABEL_FOR);
		RELATIONKEY_MAP.put(
			AccessibleRelation.LABEL_FOR,
			AccessibleConstants.RELATION_LABEL_FOR);
		RELATIONKEY_MAP.put(
			AccessibleRelation.LABELED_BY,
			AccessibleConstants.RELATION_LABELED_BY);
		RELATIONKEY_MAP.put(
			AccessibleRelation.CONTROLLER_FOR,
			AccessibleConstants.RELATION_CONTROLLER_FOR);
		RELATIONKEY_MAP.put(
			AccessibleRelation.CONTROLLED_BY,
			AccessibleConstants.RELATION_CONTROLLED_BY);
	}

	private AccessibleContext _context;
	private Accessible _accessible;

	/**
	 * wrap the given object as a ACTF <code>IAccessibleElement</code>. The 
	 * ACTF engine will invoke this constructor using a registered adaptor factory. Clients do not 
	 * typically call this constructor.
	 * 
	 * @param comp
	 */
	public JavaAccessible (Object comp) {
		if (comp instanceof Accessible) {
			_context = ((Accessible) comp).getAccessibleContext();
			_accessible = (Accessible) comp;
		}
	}

	/** {@inheritDoc} */
	public void checkIsValid () throws InvalidComponentException {
		if (_accessible == null || _context == null) {
			throw new InvalidComponentException("No accessible context"); 
		}
	}

	/** {@inheritDoc} */
	public Object getAccessibleAction () throws InvalidComponentException {
		checkIsValid();
		return _context.getAccessibleAction().toString();
	}

	/** {@inheritDoc} */
	public int getAccessibleChildCount () throws InvalidComponentException {
		checkIsValid();
		return _context.getAccessibleChildrenCount();
	}

	/** {@inheritDoc} */
	public IAccessibleElement[] getAccessibleChildren ()
		throws InvalidComponentException {
		checkIsValid();
		int count = _context.getAccessibleChildrenCount();
		IAccessibleElement[] children = new JavaAccessible[count];
		for (int c = 0; c < count; ++c) {
			children[c] = new JavaAccessible(_context.getAccessibleChild(c));
		}
		return children;
	}

	/** {@inheritDoc} */
	public IAccessibleElement getAccessibleChild (int index) throws InvalidComponentException {
		checkIsValid();
		int count = _context.getAccessibleChildrenCount();
		IAccessibleElement child = null;
		if (index < count) {
			child = new JavaAccessible(_context.getAccessibleChild(index));
		}
		return child;
	}

	/** {@inheritDoc} */
	public String getAccessibleDescription () throws InvalidComponentException {
		checkIsValid();
		return _context.getAccessibleDescription();
	}

	public Object element () {
		return _accessible;
	}

	/** {@inheritDoc} */
	public String getAccessibleKeyboardShortcut ()
		throws InvalidComponentException {
		checkIsValid();
		String res = "";
		if (_accessible instanceof AbstractButton) {
			res = "ALT+" + (char) ((AbstractButton) _accessible).getMnemonic();
		}else if (_accessible instanceof JTextComponent) {
			res = "ALT+" + ((JTextComponent) _accessible).getFocusAccelerator();
		}
		return res;
	}

	/** {@inheritDoc} */
	public String getAccessibleName () throws InvalidComponentException {
		checkIsValid();
		return _context.getAccessibleName();
	}

	/** {@inheritDoc} */
	public IAccessibleElement getAccessibleParent ()
		throws InvalidComponentException {
		checkIsValid();
		return new JavaAccessible(_context.getAccessibleParent());
	}

	/** {@inheritDoc} */
	public int getAccessibleIndexInParent () throws InvalidComponentException {
		checkIsValid();
		return SwingUtilities.getAccessibleIndexInParent((Component) _accessible);
	}
	
	/** {@inheritDoc} */
	public Map getAccessibleRelations () throws InvalidComponentException {
		checkIsValid();
		Map relations = new HashMap();
		AccessibleRelationSet set = _context.getAccessibleRelationSet();
		if (set.size() > 0) {
			AccessibleRelation[] rels = set.toArray();
			for (int r = 0; r < rels.length; ++r) {
				AccessibleRelation relation = rels[r];
				String relKey = relation.getKey();
				if (RELATIONKEY_MAP.containsKey(relKey)) {
					relKey = (String) RELATIONKEY_MAP.get(relKey);
				}
				Object[] oa = relation.getTarget();
				IAccessibleElement[] targets = new JavaAccessible[oa.length];
				for (int i = 0; i < oa.length; ++i) {
					targets[i] = new JavaAccessible(oa[i]);
				}
				relations.put(relKey, targets);
			}
		}
		return relations;
	}

	/** {@inheritDoc} */
	public String getAccessibleRole () throws InvalidComponentException {
		checkIsValid();
		String res = null;
		AccessibleRole role = _context.getAccessibleRole();
		if (role != null) {
			res = getActfRoleName(role);
		}
		return res;
	}

	/** {@inheritDoc} */
	public IAccessibleElement[] getAccessibleSelection ()
		throws InvalidComponentException {
		checkIsValid();
		AccessibleSelection selection = _context.getAccessibleSelection();
		int count = selection != null ? selection.getAccessibleSelectionCount()
				: 0;
		IAccessibleElement[] selected = new IAccessibleElement[count];
		for (int c = 0; c < count; ++c) {
			selected[c] = new JavaAccessible(selection.getAccessibleSelection(c));
		}
		return selected;
	}

	/** {@inheritDoc} */
	public Set getAccessibleState () throws InvalidComponentException {
		checkIsValid();
		HashSet res = new HashSet();
		AccessibleStateSet stateSet = _context.getAccessibleStateSet();
		AccessibleState[] stateArray = (stateSet == null ? null
				: stateSet.toArray());
		if (stateArray != null) {
			// If there is no visible state then we need to set the
			// invisible state
			boolean isVisible = false;
			for (int i = 0; i < stateArray.length; i++) {
				String name = getActfStateName(stateArray[i]);
				if (name != null) {
					res.add(name);
				}
				if (stateArray[i] == AccessibleState.VISIBLE) {
					isVisible = true;
				}
			}
			// We must always set the visible/invisible state so that there
			// is a consistent property for the client to test against
			if (isVisible == false) {
				res.add(AccessibleConstants.STATE_INVISIBLE);
			}
		}
		return res;
	}

	/** {@inheritDoc} */
	public Object getAccessibleValue () throws InvalidComponentException {
		checkIsValid();
		return _context.getAccessibleValue().getCurrentAccessibleValue().toString();
	}


	public String getAccessibleValueMin () throws InvalidComponentException {
		checkIsValid();
		AccessibleValue value = _context.getAccessibleValue();
		Number num = (value == null ? null : value.getMinimumAccessibleValue());
		return num == null ? "" : num.toString();
	}


	public String getAccessibleValueMax () throws InvalidComponentException {
		checkIsValid();
		AccessibleValue value = _context.getAccessibleValue();
		Number num = (value == null ? null : value.getMaximumAccessibleValue());
		return num == null ? "" : num.toString();
	}

	/** {@inheritDoc} */
	public IAccessibleTableElement getAccessibleTable () throws InvalidComponentException {
		checkIsValid();
		IAccessibleTableElement res = null;
		AccessibleTable table = _context.getAccessibleTable();
		if (table != null) {
			res = new JavaAccessibleTable(table);
		}
		return res;
	}

	/** {@inheritDoc} */
	public IAccessibleTextElement getAccessibleText ()
		throws InvalidComponentException {
		checkIsValid();
		IAccessibleTextElement res = null;
		AccessibleText text = _context.getAccessibleText();
		if (text != null) {
			res = new JavaAccessibleText(text);
		}
		return res;
	}

	/** {@inheritDoc} */
	public IAccessibleImageElement[] getAccessibleImage ()
		throws InvalidComponentException {
		checkIsValid();
		IAccessibleImageElement[] res = null;
		AccessibleIcon[] icon = _context.getAccessibleIcon();
		if (icon != null) {
			res = new JavaAccessibleImage[icon.length];
			for (int i = 0; i < icon.length; i++) {
				res[i] = new JavaAccessibleImage(icon[i]);
			}
		}else {
			res = new JavaAccessibleImage[0];
		}
		return res;
	}
	
	public IAccessibleEditableTextElement getAccessibleEditableText() 
		throws InvalidComponentException {
		checkIsValid();
		IAccessibleEditableTextElement res = null;
		AccessibleEditableText txt = _context.getAccessibleEditableText();
		if (txt != null) {
			res = new JavaAccessibleEditableText(txt);
		}
		return res;
	}

	private String getActfStateName (AccessibleState state) {
		String res = null;
		if (state == AccessibleState.ACTIVE) {
			res = AccessibleConstants.STATE_ACTIVE;
		}else if (state == AccessibleState.ARMED) {
			res = AccessibleConstants.STATE_ARMED;
		}else if (state == AccessibleState.BUSY) {
			res = AccessibleConstants.STATE_BUSY;
		}else if (state == AccessibleState.CHECKED) {
			res = AccessibleConstants.STATE_CHECKED;
		}else if (state == AccessibleState.COLLAPSED) {
			res = AccessibleConstants.STATE_COLLAPSED;
		}else if (state == AccessibleState.EDITABLE) {
			res = AccessibleConstants.STATE_EDITABLE;
		}else if (state == AccessibleState.ENABLED) {
			res = AccessibleConstants.STATE_ENABLED;
		}else if (state == AccessibleState.EXPANDABLE) {
			res = AccessibleConstants.STATE_EXPANDABLE;
		}else if (state == AccessibleState.EXPANDED) {
			res = AccessibleConstants.STATE_EXPANDED;
		}else if (state == AccessibleState.FOCUSABLE) {
			res = AccessibleConstants.STATE_FOCUSABLE;
		}else if (state == AccessibleState.FOCUSED) {
			res = AccessibleConstants.STATE_FOCUSED;
		}else if (state == AccessibleState.HORIZONTAL) {
			res = AccessibleConstants.STATE_HORIZONTAL;
		}else if (state == AccessibleState.ICONIFIED) {
			res = AccessibleConstants.STATE_ICONIFIED;
		}else if (String.valueOf(state) == "indeterminate") {
			res = AccessibleConstants.STATE_INDETERMINANT;
		}else if (String.valueOf(state) == "managesDescendants") {
			res = AccessibleConstants.STATE_MANAGES_DESCENDENTS;
		}else if (state == AccessibleState.MODAL) {
			res = AccessibleConstants.STATE_MODAL;
		}else if (state == AccessibleState.MULTI_LINE) {
			res = AccessibleConstants.STATE_MULTI_LINE;
		}else if (state == AccessibleState.MULTISELECTABLE) {
			res = AccessibleConstants.STATE_MULTI_SELECTABLE;
		}else if (state == AccessibleState.OPAQUE) {
			res = AccessibleConstants.STATE_OPAQUE;
		}else if (state == AccessibleState.PRESSED) {
			res = AccessibleConstants.STATE_PRESSED;
		}else if (state == AccessibleState.RESIZABLE) {
			res = AccessibleConstants.STATE_SIZEABLE;
		}else if (state == AccessibleState.SELECTABLE) {
			res = AccessibleConstants.STATE_SELECTABLE;
		}else if (state == AccessibleState.SELECTED) {
			res = AccessibleConstants.STATE_SELECTED;
		}else if (state == AccessibleState.SHOWING) {
			res = AccessibleConstants.STATE_SHOWING;
		}else if (state == AccessibleState.SINGLE_LINE) {
			res = AccessibleConstants.STATE_SINGLE_LINE;
		}else if (state == AccessibleState.TRANSIENT) {
			res = AccessibleConstants.STATE_TRANSIENT;
		}else if (String.valueOf(state) == "truncated") {
			res = AccessibleConstants.STATE_TRUNCATED;
		}else if (state == AccessibleState.VERTICAL) {
			res = AccessibleConstants.STATE_VERTICAL;
		}else if (state == AccessibleState.VISIBLE) {
			res = AccessibleConstants.STATE_VISIBLE;
		}
		return res;
	}

	private String getActfRoleName (AccessibleRole role) {
		String res = null;
		if (role == AccessibleRole.ALERT) {
			res = AccessibleConstants.ROLE_ALERT;
		}else if (role == AccessibleRole.AWT_COMPONENT) {
			res = AccessibleConstants.ROLE_AWT_COMPONENT;
		}else if (role == AccessibleRole.CANVAS) {
			res = AccessibleConstants.ROLE_CANVAS;
		}else if (role == AccessibleRole.CHECK_BOX) {
			res = AccessibleConstants.ROLE_CHECK_BOX;
		}else if (role == AccessibleRole.COLOR_CHOOSER) {
			res = AccessibleConstants.ROLE_COLOR_CHOOSER;
		}else if (role == AccessibleRole.COLUMN_HEADER) {
			res = AccessibleConstants.ROLE_COLUMN_HEADER;
		}else if (role == AccessibleRole.COMBO_BOX) {
			res = AccessibleConstants.ROLE_COMBO_BOX;
		}else if (role == AccessibleRole.DATE_EDITOR) {
			res = AccessibleConstants.ROLE_DATE_EDITOR;
		}else if (role == AccessibleRole.DESKTOP_ICON) {
			res = AccessibleConstants.ROLE_DESKTOP_ICON;
		}else if (role == AccessibleRole.DESKTOP_PANE) {
			res = AccessibleConstants.ROLE_DESKTOP_ICON;
		}else if (role == AccessibleRole.DIALOG) {
			res = AccessibleConstants.ROLE_DIALOG;
		}else if (role == AccessibleRole.DIRECTORY_PANE) {
			res = AccessibleConstants.ROLE_DIRECTORY_PANE;
		}else if (String.valueOf(role) == "editbar") {
			res = AccessibleConstants.ROLE_EDIT_BAR;
		}else if (role == AccessibleRole.FILE_CHOOSER) {
			res = AccessibleConstants.ROLE_FILE_CHOOSER;
		}else if (role == AccessibleRole.FILLER) {
			res = AccessibleConstants.ROLE_FILLER;
		}else if (role == AccessibleRole.FONT_CHOOSER) {
			res = AccessibleConstants.ROLE_FONT_CHOOSER;
		}else if (String.valueOf(role) == "footer") {
			res = AccessibleConstants.ROLE_FOOTER;
		}else if (role == AccessibleRole.FRAME) {
			res = AccessibleConstants.ROLE_FRAME;
		}else if (role == AccessibleRole.GLASS_PANE) {
			res = AccessibleConstants.ROLE_GLASS_PANE;
		}else if (role == AccessibleRole.GROUP_BOX) {
			res = AccessibleConstants.ROLE_GROUP_BOX;
		}else if (String.valueOf(role) == "header") {
			res = AccessibleConstants.ROLE_HEADER;
		}else if (role == AccessibleRole.HYPERLINK) {
			res = AccessibleConstants.ROLE_HYPERLINK;
		}else if (role == AccessibleRole.ICON) {
			res = AccessibleConstants.ROLE_ICON;
		}else if (role == AccessibleRole.INTERNAL_FRAME) {
			res = AccessibleConstants.ROLE_INTERNAL_FRAME;
		}else if (role == AccessibleRole.LABEL) {
			res = AccessibleConstants.ROLE_LABEL;
		}else if (role == AccessibleRole.LAYERED_PANE) {
			res = AccessibleConstants.ROLE_LAYERED_PANE;
		}else if (role == AccessibleRole.LIST) {
			res = AccessibleConstants.ROLE_LIST;
		}else if (role == AccessibleRole.LIST_ITEM) {
			res = AccessibleConstants.ROLE_LIST_ITEM;
		}else if (role == AccessibleRole.MENU) {
			res = AccessibleConstants.ROLE_MENU;
		}else if (role == AccessibleRole.MENU_BAR) {
			res = AccessibleConstants.ROLE_MENU_BAR;
		}else if (role == AccessibleRole.MENU_ITEM) {
			res = AccessibleConstants.ROLE_MENU_ITEM;
		}else if (role == AccessibleRole.OPTION_PANE) {
			res = AccessibleConstants.ROLE_OPTION_PANE;
		}else if (role == AccessibleRole.PAGE_TAB) {
			res = AccessibleConstants.ROLE_PAGE_TAB;
		}else if (role == AccessibleRole.PAGE_TAB_LIST) {
			res = AccessibleConstants.ROLE_PAGE_TAB_LIST;
		}else if (role == AccessibleRole.PANEL) {
			res = AccessibleConstants.ROLE_PANEL;
		}else if (String.valueOf(role) == "paragraph") {
			res = AccessibleConstants.ROLE_PARAGRAPH;
		}else if (role == AccessibleRole.PASSWORD_TEXT) {
			res = AccessibleConstants.ROLE_PASSWORD_TEXT;
		}else if (role == AccessibleRole.POPUP_MENU) {
			res = AccessibleConstants.ROLE_POPUP_MENU;
		}else if (role == AccessibleRole.PROGRESS_BAR) {
			res = AccessibleConstants.ROLE_PROGRESS_BAR;
		}else if (String.valueOf(role) == "progressMonitor") {
			res = AccessibleConstants.ROLE_PROGRESS_MONITOR;
		}else if (role == AccessibleRole.PUSH_BUTTON) {
			res = AccessibleConstants.ROLE_PUSH_BUTTON;
		}else if (role == AccessibleRole.RADIO_BUTTON) {
			res = AccessibleConstants.ROLE_RADIO_BUTTON;
		}else if (role == AccessibleRole.ROOT_PANE) {
			res = AccessibleConstants.ROLE_ROOT_PANE;
		}else if (role == AccessibleRole.ROW_HEADER) {
			res = AccessibleConstants.ROLE_ROW_HEADER;
		}else if (String.valueOf(role) == "ruler") {
			res = AccessibleConstants.ROLE_RULER;
		}else if (role == AccessibleRole.SCROLL_BAR) {
			res = AccessibleConstants.ROLE_SCROLL_BAR;
		}else if (role == AccessibleRole.SCROLL_PANE) {
			res = AccessibleConstants.ROLE_SCROLL_PANE;
		}else if (role == AccessibleRole.SEPARATOR) {
			res = AccessibleConstants.ROLE_SEPARATOR;
		}else if (role == AccessibleRole.SLIDER) {
			res = AccessibleConstants.ROLE_SLIDER;
		}else if (role == AccessibleRole.SPIN_BOX) {
			res = AccessibleConstants.ROLE_SPIN_BOX;
		}else if (role == AccessibleRole.SPLIT_PANE) {
			res = AccessibleConstants.ROLE_SPLIT_PANE;
		}else if (role == AccessibleRole.STATUS_BAR) {
			res = AccessibleConstants.ROLE_STATUS_BAR;
		}else if (role == AccessibleRole.SWING_COMPONENT) {
			res = AccessibleConstants.ROLE_SWING_COMPONENT;
		}else if (role == AccessibleRole.TABLE) {
			res = AccessibleConstants.ROLE_TABLE;
		}else if (role == AccessibleRole.TEXT) {
			res = AccessibleConstants.ROLE_TEXT;
		}else if (role == AccessibleRole.TOGGLE_BUTTON) {
			res = AccessibleConstants.ROLE_TOGGLE_BUTTON;
		}else if (role == AccessibleRole.TOOL_BAR) {
			res = AccessibleConstants.ROLE_TOOL_BAR;
		}else if (role == AccessibleRole.TOOL_TIP) {
			res = AccessibleConstants.ROLE_TOOL_TIP;
		}else if (role == AccessibleRole.TREE) {
			res = AccessibleConstants.ROLE_TREE;
		}else if (role == AccessibleRole.UNKNOWN) {
			res = AccessibleConstants.ROLE_UNKNOWN;
		}else if (role == AccessibleRole.VIEWPORT) {
			res = AccessibleConstants.ROLE_VIEW_PORT;
		}else if (role == AccessibleRole.WINDOW) {
			res = AccessibleConstants.ROLE_WINDOW;
		}
		return res;
	}
	public Rectangle getAccessibleLocation() throws InvalidComponentException {
		checkIsValid();
		return _context.getAccessibleComponent().getBounds();
	}
	
	public boolean equals (Object other) {
		return _accessible.equals(other);
	}
	
	public int hashCode () {
		return _accessible.hashCode();
	}

}
