/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.accservice.mozdom;

import java.awt.Rectangle;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.eclipse.actf.accservice.core.AccessibleConstants;
import org.eclipse.actf.accservice.core.IAccessibleElement;
import org.eclipse.actf.model.InvalidComponentException;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.mozilla.interfaces.nsIAccessibilityService;
import org.mozilla.interfaces.nsIAccessible;
import org.mozilla.interfaces.nsIAccessibleSelectable;
import org.mozilla.interfaces.nsIArray;
import org.mozilla.interfaces.nsIDOMNode;
import org.mozilla.interfaces.nsISupports;
import org.mozilla.xpcom.Mozilla;
import org.mozilla.xpcom.XPCOMException;


public class MozillaAccessible implements IAccessibleElement
{

	private static final Map RELATIONKEY_MAP = new HashMap();
	static {
		RELATIONKEY_MAP.put(
			new Long(nsIAccessible.RELATION_CONTROLLER_FOR),
			AccessibleConstants.RELATION_CONTROLLER_FOR);
		RELATIONKEY_MAP.put(
			new Long(nsIAccessible.RELATION_CONTROLLED_BY),
			AccessibleConstants.RELATION_CONTROLLED_BY);
		RELATIONKEY_MAP.put(
			new Long(nsIAccessible.RELATION_LABEL_FOR),
			AccessibleConstants.RELATION_LABEL_FOR);
		RELATIONKEY_MAP.put(
			new Long(nsIAccessible.RELATION_LABELLED_BY),
			AccessibleConstants.RELATION_LABELED_BY);
		RELATIONKEY_MAP.put(
			new Long(nsIAccessible.RELATION_NODE_CHILD_OF),
			AccessibleConstants.RELATION_NODE_CHILD_OF);
		RELATIONKEY_MAP.put(
			new Long(nsIAccessible.RELATION_MEMBER_OF),
			AccessibleConstants.RELATION_MEMBER_OF);
		RELATIONKEY_MAP.put(
			new Long(nsIAccessible.RELATION_FLOWS_TO),
			AccessibleConstants.RELATION_FLOWS_TO);
		RELATIONKEY_MAP.put(
			new Long(nsIAccessible.RELATION_FLOWS_FROM),
			AccessibleConstants.RELATION_FLOWS_FROM);
		RELATIONKEY_MAP.put(
			new Long(nsIAccessible.RELATION_EMBEDS),
			AccessibleConstants.RELATION_EMBEDS);
		RELATIONKEY_MAP.put(
			new Long(nsIAccessible.RELATION_EMBEDDED_BY),
			AccessibleConstants.RELATION_EMBEDDED_BY);
		RELATIONKEY_MAP.put(
			new Long(nsIAccessible.RELATION_PARENT_WINDOW_OF),
			AccessibleConstants.RELATION_PARENT_WINDOW_OF);
		RELATIONKEY_MAP.put(
			new Long(nsIAccessible.RELATION_SUBWINDOW_OF),
			AccessibleConstants.RELATION_SUBWINDOW_OF);
		RELATIONKEY_MAP.put(
			new Long(nsIAccessible.RELATION_DESCRIPTION_FOR),
			AccessibleConstants.RELATION_DESCRIPTION_FOR);
		RELATIONKEY_MAP.put(
			new Long(nsIAccessible.RELATION_DESCRIBED_BY),
			AccessibleConstants.RELATION_DESCRIBED_BY);
	}

	// These values were obtained from org.eclipse.swt.accessibility.ACC
	private static final int MSAA_ROLE_CLIENT_AREA = 0xa;

	private static final int MSAA_ROLE_WINDOW = 0x9;

	private static final int MSAA_ROLE_MENU_BAR = 0x2;

	private static final int MSAA_ROLE_MENU = 0xb;

	private static final int MSAA_ROLE_MENU_ITEM = 0xc;

	private static final int MSAA_ROLE_SEPARATOR = 0x15;

	private static final int MSAA_ROLE_TOOL_TIP = 0xd;

	private static final int MSAA_ROLE_SCROLL_BAR = 0x3;

	private static final int MSAA_ROLE_DIALOG = 0x12;

	private static final int MSAA_ROLE_LABEL = 0x29;

	private static final int MSAA_ROLE_PUSH_BUTTON = 0x2b;

	private static final int MSAA_ROLE_CHECK_BUTTON = 0x2c;

	private static final int MSAA_ROLE_RADIO_BUTTON = 0x2d;

	private static final int MSAA_ROLE_COMBO_BOX = 0x2e;

	private static final int MSAA_ROLE_TEXT = 0x2a;

	private static final int MSAA_ROLE_TOOL_BAR = 0x16;

	private static final int MSAA_ROLE_LIST = 0x21;

	private static final int MSAA_ROLE_LIST_ITEM = 0x22;

	private static final int MSAA_ROLE_TABLE = 0x18;

	private static final int MSAA_ROLE_TABLE_CELL = 0x1d;

	private static final int MSAA_ROLE_TABLE_COLUMN_HEADER = 0x19;

	private static final int MSAA_ROLE_TABLE_ROW_HEADER = 0x1a;

	private static final int MSAA_ROLE_TREE = 0x23;

	private static final int MSAA_ROLE_TREE_ITEM = 0x24;

	private static final int MSAA_ROLE_TAB_FOLDER = 0x3c;

	private static final int MSAA_ROLE_TAB_ITEM = 0x25;

	private static final int MSAA_ROLE_PROGRESS_BAR = 0x30;

	private static final int MSAA_ROLE_SLIDER = 0x33;

	private static final int MSAA_ROLE_LINK = 0x1e;

	private static nsIAccessibilityService accService;

	private nsIAccessible _accessible;

	private nsIDOMNode _node;

	public MozillaAccessible (Object element) {
		if (accService == null) {
			try {
				accService = (nsIAccessibilityService) Mozilla.getInstance().getComponentManager().createInstanceByContractID(
					"@mozilla.org/accessibilityService;1", null,
					nsIAccessibilityService.NS_IACCESSIBILITYSERVICE_IID);
			}catch (XPCOMException e) {
				throw new RuntimeException("Could not retreave nsIAccessibilityService", e);
			}
		}
		if (accService != null && element instanceof nsISupports) {
			nsIDOMNode node = null;
			try {
				node = (nsIDOMNode) ((nsISupports) element).queryInterface(nsIDOMNode.NS_IDOMNODE_IID);
				_accessible = accService.getAccessibleFor(node);
			}catch (Exception e) {
				_accessible = null;
				// Why are we not passing the exception in the println?
				// Some nodes do not have accessible objects so we just want
				// To report this on a single line without putting a stack trace in the log
				LoggingUtil.println(LoggingUtil.WARNINGS, "Could not create accessible for " + (node != null ? node.getNodeName() : "element"));
			}
			_node = node;
		}
	}

	/**
	 * wrap the given object as a ACTF <code>IAccessibleElement</code>. The 
	 * ACTF engine will invoke this constructor using a registered adaptor factory. Clients do not 
	 * typically call this constructor.
	 * 
	 * @param acc
	 */
	public MozillaAccessible (nsIAccessible acc) {
		_accessible = acc;
	}

	/** {@inheritDoc} */
	public void checkIsValid () throws InvalidComponentException {
		if (_accessible == null) {
			throw new InvalidComponentException("Invalid nsIAccessible"); 
		}
	}

	/** {@inheritDoc} */
	public String getAccessibleAction () throws InvalidComponentException {
		checkIsValid();
		short count = _accessible.getNumActions();
		StringBuffer sb = new StringBuffer();
		for (short s = 0; s < count; ++s) {
			sb.append(_accessible.getActionName(s));
			if (s == 0) {
				sb.append("[default]");
			}
			if (s < count - 1) {
				sb.append(',');
			}
		}
		return sb.toString();
	}

	/** {@inheritDoc} */
	public int getAccessibleChildCount () throws InvalidComponentException {
		checkIsValid();
		return _accessible.getChildCount();
	}

	/** {@inheritDoc} */
	public IAccessibleElement[] getAccessibleChildren ()
		throws InvalidComponentException {
		checkIsValid();
		int count = _accessible.getChildCount();
		IAccessibleElement[] children = new MozillaAccessible[count];
		for (int c = 0; c < count; ++c) {
			children[c] = new MozillaAccessible(_accessible.getChildAt(c));
		}
		return children;
	}

	/** {@inheritDoc} */
	public IAccessibleElement getAccessibleChild (int index) throws InvalidComponentException {
		checkIsValid();
		int count = _accessible.getChildCount();
		IAccessibleElement child = null;
		if (index < count) {
			child = new MozillaAccessible(_accessible.getChildAt(index));
		}
		return child;
	}

	public String getAccessibleDescription () throws InvalidComponentException {
		checkIsValid();
		return _accessible.getDescription();
	}

	/** {@inheritDoc} */
	public String getAccessibleKeyboardShortcut ()
		throws InvalidComponentException {
		checkIsValid();
		return _accessible.getKeyboardShortcut();
	}

	/** {@inheritDoc} */
	public String getAccessibleName () throws InvalidComponentException {
		checkIsValid();
		return _accessible.getName();
	}

	/** {@inheritDoc} */
	public IAccessibleElement getAccessibleParent ()
		throws InvalidComponentException {
		checkIsValid();
		return new MozillaAccessible(_accessible.getParent());
	}

	/** {@inheritDoc} */
	public int getAccessibleIndexInParent ()
		throws InvalidComponentException {
		checkIsValid();
		return _accessible.getIndexInParent();
	}

	public Map getAccessibleRelations () throws InvalidComponentException {
		checkIsValid();
		HashMap relMap = new HashMap();
		for (Iterator iter = RELATIONKEY_MAP.keySet().iterator(); iter.hasNext();) {
			Long relType = (Long) iter.next();
			nsIAccessible acc = _accessible.getAccessibleRelated(relType.longValue());
			if (acc != null) {
				relMap.put(relType, acc);
			}
		}
		return relMap;
	}

	/** {@inheritDoc} */
	public String getAccessibleRole () throws InvalidComponentException {
		checkIsValid();
		return getActfRoleName(_accessible.getFinalRole());
	}

	/** {@inheritDoc} */
	public IAccessibleElement[] getAccessibleSelection ()
		throws InvalidComponentException {
		checkIsValid();
		nsIAccessibleSelectable accSelectable = (nsIAccessibleSelectable) _accessible.queryInterface(nsIAccessibleSelectable.NS_IACCESSIBLESELECTABLE_IID);
		int count = accSelectable.getSelectionCount();
		IAccessibleElement[] selections = new MozillaAccessible[count];
		nsIArray selectables = accSelectable.getSelectedChildren();
		int length = (int) selectables.getLength();
		for (int s = 0; s < length; ++s) {
			selections[s] = new MozillaAccessible((nsIAccessible) selectables.queryElementAt(
				s, nsIAccessible.NS_IACCESSIBLE_IID));
		}
		return selections;
	}

	/** {@inheritDoc} */
	public Set getAccessibleState () throws InvalidComponentException {
		checkIsValid();
		HashSet res = new HashSet();
		long state = _accessible.getFinalState();
		// We must always set the visible/invisible state so that there
		// is a consistent property for the client to test against
		if ((state & nsIAccessible.STATE_INVISIBLE) != 0) {
			res.add(AccessibleConstants.STATE_INVISIBLE);
		}else {
			res.add(AccessibleConstants.STATE_VISIBLE);
		}
		if ((state & nsIAccessible.STATE_BUSY) != 0) {
			res.add(AccessibleConstants.STATE_BUSY);
		}
		if ((state & nsIAccessible.STATE_CHECKED) != 0) {
			res.add(AccessibleConstants.STATE_CHECKED);
		}
		if ((state & nsIAccessible.STATE_CHECKABLE) != 0) {
			res.add(AccessibleConstants.STATE_CHECKABLE);
		}
		if ((state & nsIAccessible.STATE_COLLAPSED) != 0) {
			res.add(AccessibleConstants.STATE_COLLAPSED);
		}
		if ((state & nsIAccessible.STATE_EXPANDED) != 0) {
			res.add(AccessibleConstants.STATE_EXPANDABLE);
		}
		if ((state & nsIAccessible.STATE_FOCUSABLE) != 0) {
			res.add(AccessibleConstants.STATE_FOCUSABLE);
		}
		if ((state & nsIAccessible.STATE_FOCUSED) != 0) {
			res.add(AccessibleConstants.STATE_FOCUSED);
		}
		if ((state & nsIAccessible.STATE_HOTTRACKED) != 0) {
			res.add(AccessibleConstants.STATE_HOT_TRACKED);
		}
		if ((state & nsIAccessible.STATE_LINKED) != 0) {
			res.add(AccessibleConstants.STATE_LINKED);
		}
		if ((state & nsIAccessible.STATE_DEFAULT) != 0) {
			res.add(AccessibleConstants.STATE_NORMAL);
		}
		if ((state & nsIAccessible.STATE_OFFSCREEN) != 0) {
			res.add(AccessibleConstants.STATE_OFF_SCREEN);
		}
		if ((state & nsIAccessible.STATE_PRESSED) != 0) {
			res.add(AccessibleConstants.STATE_PRESSED);
		}
		if ((state & nsIAccessible.STATE_INVALID) != 0) {
			res.add(AccessibleConstants.STATE_INVALID);
		}
		if ((state & nsIAccessible.STATE_REQUIRED) != 0) {
			res.add(AccessibleConstants.STATE_REQUIRED);
		}
		if ((state & nsIAccessible.STATE_READONLY) != 0) {
			res.add(AccessibleConstants.STATE_READ_ONLY);
		}
		if ((state & nsIAccessible.STATE_SELECTABLE) != 0) {
			res.add(AccessibleConstants.STATE_SELECTABLE);
		}
		if ((state & nsIAccessible.STATE_SELECTED) != 0) {
			res.add(AccessibleConstants.STATE_SELECTED);
		}
		if ((state & nsIAccessible.STATE_SIZEABLE) != 0) {
			res.add(AccessibleConstants.STATE_SIZEABLE);
		}
		return res;
	}

	/** {@inheritDoc} */
	public String getAccessibleValue () throws InvalidComponentException {
		checkIsValid();
		return _accessible.getFinalValue();
	}

	/** {@inheritDoc} */
	public Object element () {
		return _node;
	}

	private String getActfRoleName (long role) {
		String res = null;
		if (role == MSAA_ROLE_CLIENT_AREA) {
			res = AccessibleConstants.ROLE_CLIENT_AREA;
		}else if (role == MSAA_ROLE_WINDOW) {
			res = AccessibleConstants.ROLE_WINDOW;
		}else if (role == MSAA_ROLE_MENU_BAR) {
			res = AccessibleConstants.ROLE_MENU_BAR;
		}else if (role == MSAA_ROLE_MENU) {
			res = AccessibleConstants.ROLE_MENU;
		}else if (role == MSAA_ROLE_MENU_ITEM) {
			res = AccessibleConstants.ROLE_MENU_ITEM;
		}else if (role == MSAA_ROLE_SEPARATOR) {
			res = AccessibleConstants.ROLE_SEPARATOR;
		}else if (role == MSAA_ROLE_TOOL_TIP) {
			res = AccessibleConstants.ROLE_TOOL_TIP;
		}else if (role == MSAA_ROLE_SCROLL_BAR) {
			res = AccessibleConstants.ROLE_SCROLL_BAR;
		}else if (role == MSAA_ROLE_DIALOG) {
			res = AccessibleConstants.ROLE_DIALOG;
		}else if (role == MSAA_ROLE_LABEL) {
			res = AccessibleConstants.ROLE_LABEL;
		}else if (role == MSAA_ROLE_PUSH_BUTTON) {
			res = AccessibleConstants.ROLE_PUSH_BUTTON;
		}else if (role == MSAA_ROLE_CHECK_BUTTON) {
			res = AccessibleConstants.ROLE_CHECK_BOX;
		}else if (role == MSAA_ROLE_RADIO_BUTTON) {
			res = AccessibleConstants.ROLE_RADIO_BUTTON;
		}else if (role == MSAA_ROLE_COMBO_BOX) {
			res = AccessibleConstants.ROLE_COMBO_BOX;
		}else if (role == MSAA_ROLE_TEXT) {
			res = AccessibleConstants.ROLE_TEXT;
		}else if (role == MSAA_ROLE_TOOL_BAR) {
			res = AccessibleConstants.ROLE_TOOL_BAR;
		}else if (role == MSAA_ROLE_LIST) {
			res = AccessibleConstants.ROLE_LIST;
		}else if (role == MSAA_ROLE_LIST_ITEM) {
			res = AccessibleConstants.ROLE_LIST_ITEM;
		}else if (role == MSAA_ROLE_TABLE) {
			res = AccessibleConstants.ROLE_TABLE;
		}else if (role == MSAA_ROLE_TABLE_CELL) {
			res = AccessibleConstants.ROLE_TABLECELL;
		}else if (role == MSAA_ROLE_TABLE_COLUMN_HEADER) {
			res = AccessibleConstants.ROLE_TABLECOLUMNHEADER;
		}else if (role == MSAA_ROLE_TABLE_ROW_HEADER) {
			res = AccessibleConstants.ROLE_TABLEROWHEADER;
		}else if (role == MSAA_ROLE_TREE) {
			res = AccessibleConstants.ROLE_TREE;
		}else if (role == MSAA_ROLE_TREE_ITEM) {
			res = AccessibleConstants.ROLE_TREEITEM;
		}else if (role == MSAA_ROLE_TAB_FOLDER) {
			res = AccessibleConstants.ROLE_TABFOLDER;
		}else if (role == MSAA_ROLE_TAB_ITEM) {
			res = AccessibleConstants.ROLE_TABITEM;
		}else if (role == MSAA_ROLE_PROGRESS_BAR) {
			res = AccessibleConstants.ROLE_PROGRESS_BAR;
		}else if (role == MSAA_ROLE_SLIDER) {
			res = AccessibleConstants.ROLE_SLIDER;
		}else if (role == MSAA_ROLE_LINK) {
			res = AccessibleConstants.ROLE_LINK;
		}
		return res;
	}

	public Rectangle getAccessibleLocation() throws InvalidComponentException {
		checkIsValid();
		//return _accessible.getKeyboardShortcut();
		int[] x = new int[1];
		int[] y = new int[1];
		int[] wid = new int[1];
		int[] ht = new int[1];
		_accessible.getBounds(x, y, wid, ht);
		
		return new Rectangle(x[1], y[1], wid[1], ht[1]);
	}

}
