/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.model.javapp;

import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.List;

import org.eclipse.actf.core.adapt.DefaultAdaptorFactory;
import org.eclipse.actf.core.adapt.IAdaptor;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.model.AbstractGuiModel;
import org.eclipse.actf.model.IGuiModel;
import org.eclipse.actf.model.javapp.locate.SwtComponentLocator;
import org.eclipse.actf.model.javapp.traverse.SwtNodeItemFilter;
import org.eclipse.actf.model.javapp.traverse.SwtTreeNodeWalker;
import org.eclipse.actf.model.locate.INodeLocator;
import org.eclipse.actf.model.traverse.ITreeNodeWalker;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.actf.util.resources.ClassLoaderCache;

/**
 * implementation for the Eclipse SWT GUI model
 * @author Mike Squillace
 */
public class SwtGuiModel extends AbstractGuiModel
{

	public static final String[] FOCUSABLE_COMPOSITE_NAMES = new String[] {
			"org.eclipse.swt.widgets.Combo", "org.eclipse.swt.custom.CCombo",
			"org.eclipse.swt.custom.CTabFolder",
			"org.eclipse.ui.dialogs.FilteredList",
			"org.eclipse.swt.widgets.Table",
			"org.eclipse.swt.custom.TableTree", "org.eclipse.swt.widgets.Tree"};

	public static final List FOCUSABLE_COMPOSITE_LIST = Arrays.asList(FOCUSABLE_COMPOSITE_NAMES);
	public static final String SWT_MODEL = "swt";
	public static final String SWT_RUNTIME_HOME_KEY = "swt.runtime.home";
		
	private static final String[] PACKAGE_LIST = new String[] {
			"java.lang", "org.eclipse.swt", "org.eclipse.swt.events",
			"org.eclipse.swt.widgets", "org.eclipse.swt.custom"
	};

	public static final String SWT_ADAPTOR_CLASS = "org.eclipse.actf.accservice.javapp.adapt.SwtAccessibilityAdaptor";
	private static Class ctrlType;

	protected ClassLoaderCache clCache = ClassLoaderCache.getDefault();

	private Object _display;

	/**
	 * create a new SwtGuiModel
	 */
	public SwtGuiModel () {
		super(SWT_MODEL);
		ctrlType = org.eclipse.swt.widgets.Control.class; //clCache.classForName("org.eclipse.swt.widgets.Control");
		try {
			_display = org.eclipse.swt.widgets.Display.getDefault();
		}catch (Exception e) {
			LoggingUtil.println(
				LoggingUtil.WARNINGS,
				"Could not find display in SWT IModel ctor");
		}
		
		treeNodeWalker = null;
		Class baseTypeCls = ClassLoaderCache.getDefault().classForName(getBaseType());
		Class adaptorCls = ClassLoaderCache.getDefault().classForName(SWT_ADAPTOR_CLASS);
		IAdaptor adaptor = null;
		try {
			adaptor = (IAdaptor) adaptorCls.newInstance();
		} catch (Exception e) {
			LoggingUtil.println(
					IReporter.SYSTEM_NONFATAL, "Cannot get adaptor class for SWT Model", e);
		}
		if (adaptor != null){
			DefaultAdaptorFactory.getInstance().registerAdaptor(baseTypeCls,adaptor);
		}
	}

	/** {@inheritDoc} */
	public String getDefaultAliasPrefix () {
		return "org.eclipse.swt.widgets";
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * Includes packages:
	 * <p>
	 * <ul>
	 * <li>java.lang
	 * <li>org.eclipse.swt
	 * <li>org.eclipse.swt.events
	 * <li>org.eclipse.swt.widgets
	 * <li>org.eclipse.swt.layout
	 * <li>org.eclipse.swt.custom
	 * </ul>
	 */
	public String[] getPackageNames () {
		return PACKAGE_LIST;
	}

	/**
	 * return <code>true</code> since SWT components must be created and added
	 * to GUIs upon their creation
	 * 
	 * @return <code>true</code>
	 */
	public boolean performsLinkOnCreation () {
		return true;
	}

	/**
	 * @return <code>true</code>
	 */
	public boolean isTopDown () {
		return true;
	}

	/**
	 * An object must be an instance of a class that inherets from
	 * <code>org.eclipse.swt.widgets.Shell</code> in order to serve as a
	 * top-level component in a SWT GUI
	 * 
	 * @param c --
	 *            class to be tested
	 * @return <code>true</code> if the given class inherits from
	 *         <code>org.eclipse.swt.widgets.Shell</code>
	 */
	public boolean isGuiRootType (Class c) {
		return clCache.isAssignableFrom("org.eclipse.swt.widgets.Shell", c);
	}

	/**
	 * An object must be an instance of a class that inherets from
	 * <code>org.eclipse.swt.widgets.Shell</code> in order to serve as a
	 * top-level component in a SWT GUI
	 * 
	 * @param o --
	 *            object to be tested
	 * @return <code>true</code> if object is an instance of a class that
	 *         inherets from <code>org.eclipse.swt.widgets.Shell</code>
	 */
	public boolean isGuiRoot (Object o) {
		return isGuiRootType(o.getClass());
	}

	/**
	 * returns whether or not the currently executing thread is the UI thread.
	 * In SWT, the UI thread is the thread that created the
	 * <code>org.eclipse.swt.widgets.Display</code> instance.
	 * 
	 * @return <code>true</code> if the current thread is the UI thread,
	 *         <code>false</code> otherwise
	 * @see IGuiModel#isUIThread()
	 */
	public boolean isUIThread () {
		Thread displayThread = null;
		try {
			invokeGetter(_display, "getThread");
		}catch (Exception e) {
		}
		return displayThread == null || Thread.currentThread() == displayThread;
	}

	/** {@inheritDoc} */
	public boolean isVisible (final Object component) {
		final boolean[] result = new boolean[] {true};
		invokeOnUIThread(new Runnable() {

			public void run () {
				Object widget = component;
				boolean isControl = ctrlType.isAssignableFrom(component.getClass());
				boolean isItem = clCache.isInstanceOf(
					"org.eclipse.swt.widgets.Item", component);
				if (isItem) {
					try {
						widget = invokeGetter(component, "getParent");
					}catch (Exception e) {
					}
				}
				if (widget != null) {
					boolean isDisposed = isDisposed(widget);
					boolean isVisible = internalIsVisible(widget);
					Object parentShell = null;
					try {
						parentShell = invokeGetter(widget, "getShell");
					}catch (Exception e) {
					}
					boolean isParentShellVisible = parentShell != null
							&& internalIsVisible(parentShell);
					result[0] = !isControl
							|| (!isDisposed && isVisible && isParentShellVisible);
				}
			}
		});
		return result[0];
	}

	/**
	 * executes the given Runnable from within the UI thread. This method will
	 * typically be called if <code>isUIThread</code> returns
	 * <code>false</code>.
	 * 
	 * @param runnable -
	 *            Runnable to be invoked in UI thread
	 * @see #isUIThread()
	 */
	public void invokeOnUIThread (Runnable runnable) {
		try {
			Method meth = _display.getClass().getMethod("isDisposed", (Class[]) null);
			boolean isDisposed = ((Boolean) meth.invoke(_display, (Object[]) null)).booleanValue();
			if (!isDisposed) {
				meth = _display.getClass().getMethod(
					"syncExec", new Class[] {Runnable.class});
				meth.invoke(_display, new Object[] {runnable});
			}
		}catch (Throwable e) {
			LoggingUtil.println(
				IReporter.SYSTEM_FATAL, "Cannot invoke runnable on UI thread", e);
		}
	}

	/** {@inheritDoc} */
	public void asyncInvokeOnUIThread (Runnable runnable) {
		try {
			Method meth = _display.getClass().getMethod(
				"asyncExec", new Class[] {Runnable.class});
			meth.invoke(_display, new Object[] {runnable});
		}catch (Exception e) {
		}
	}

	/** {@inheritDoc} */
	public boolean requestFocusFor (final Object comp) {
		boolean res = false;
		if (clCache.isInstanceOf("org.eclipse.swt.widgets.Shell", comp)
				&& !isDisposed(comp)) {
			try {
				final Class c = org.eclipse.swt.widgets.Shell.class;
				if (c != null) {
					invokeOnUIThread(new Runnable() {

						public void run () {
							try {
								c.getMethod("setActive", (Class[]) null).invoke(
									comp, (Object[]) null);
							}catch (Exception e) {
								LoggingUtil.println(IReporter.SYSTEM_NONFATAL, e);
							}
						}
					});
				}
			}catch (Exception e) {
				LoggingUtil.println(IReporter.SYSTEM_NONFATAL, e);
			}
		}
		if (!FOCUSABLE_COMPOSITE_LIST.contains(comp.getClass().getName())
				&& clCache.isInstanceOf(
					"org.eclipse.swt.widgets.Composite", comp)
				&& !isDisposed(comp)) {
			Object[] children = null;
			try {
				children = (Object[]) invokeGetter(comp, "getChildren");
			}catch (Exception e) {
			}
			if (children == null || children.length == 0) {
				try {
					children = (Object[]) invokeGetter(comp, "getItems");
				}catch (Exception e) {
				}
			}
			for (int c = 0; children != null && !res & c < children.length; ++c) {
				res = requestFocusFor(children[c]);
			}
		}else if (ctrlType.isAssignableFrom(comp.getClass())
				&& !isDisposed(comp)) {
			try {
				res = ((Boolean) ctrlType.getMethod("setFocus", (Class[]) null).invoke(
					comp, (Object[]) null)).booleanValue();
			}catch (Exception e) {
			}
		}else if (clCache.isInstanceOf("org.eclipse.swt.widgets.Item", comp)) {
			try {
				Object ctl = SwtGuiModel.this.invokeGetter(
					comp, "getControl");
				res = ((Boolean) ctrlType.getMethod("setFocus", (Class[]) null).invoke(
					ctl, (Object[]) null)).booleanValue();
			}catch (Exception e) {
			}
		}
		return res;
	}

	/**
	 * invoke the getter method on the specified object. The method name
	 * must name a method that has no formal parameters.
	 * 
	 * <p>Note: If the specified method is found and returns a  <code>boolean</code>, then
	 * an unsuccessful invocation will result in a return value of <code>false</code>.
	 * 
	 * @param obj - receiver
	 * @param name - method name
	 * @return result of invocation
	 * @throws Exception
	 */
	public Object invokeGetter (Object obj, String name) throws Exception {
		return invokeGetter(obj, name, null);
	}

	/**
	 * invoke the specified getter method on the given object, passing it the single parameter.

	 * <p>Note: If the specified method is found and returns a  <code>boolean</code>, then
	 * an unsuccessful invocation will result in a return value of <code>false</code>.
	 *      * 
	 * @param obj - the receiver
	 * @param name - the method name
	 * @param arg - single argument to method
	 * @return result of invokation
	 * @throws Exception
	 */
	public Object invokeGetter (final Object obj, final String name,
								final Object arg) throws Exception {
		final Object[] result = new Object[1];
		final Exception[] t = new Exception[1];
		invokeOnUIThread(new Runnable() {

			public void run () {
				Method meth = null;
				if (obj != null && !isDisposed(obj)) {
					try {
						meth = obj.getClass().getMethod(
							name,
							arg == null ? new Class[0]
									: new Class[] {arg.getClass()});
						result[0] = meth.invoke(
							obj, arg == null ? new Object[0]
									: new Object[] {arg});
					}catch (Exception e) {
						//Utils.println(Reporter.SYSTEM_NONFATAL, "Could not execute method " + (meth == null ? "on object " + obj : meth.toString()), e);
						t[0] = e;
					}
				}
			}
		});
		if (t[0] != null) { throw t[0]; }
		return result[0];
	}

	/**
	 * returns whether or not the component is visible. If an exception occurs, the
	 * object is assumed to be not visible and <code>true</code> is returned.
	 * 
	 * @param comp SWT control
	 * @return <code>false</code> if and only if no exceptions occur and component
	 * is not, in fact, visible
	 */
	protected boolean internalIsVisible (Object comp) {
		boolean isVisible = false;
		try {
			Object res = invokeGetter(comp, "isVisible");
			isVisible = res != null && res instanceof Boolean
					&& ((Boolean) res).booleanValue();
		}catch (Exception e) {
		}
		return isVisible;
	}

	/**
	 * returns whether or not the component is disposed. If an exception occurs, the
	 * object is assumed to be disposed and <code>true</code> is returned.
	 * 
	 * @param comp SWT control
	 * @return <code>false</code> if and only if no exceptions occur and component
	 * is not, in fact, disposed
	 */
	public boolean isDisposed (Object comp) {
		boolean result = true;
		try {
			Method meth = comp.getClass().getMethod("isDisposed", (Class[]) null);
			Object res = meth.invoke(comp, (Object[]) null);
			result = res != null && res instanceof Boolean
					&& ((Boolean) res).booleanValue();
		}catch (Exception e) {
		}
		return result;
	}

	/** {@inheritDoc} */
	public boolean isValid (Object comp) {
		boolean res = super.isValid(comp);
		res &= !isDisposed(comp);
		return res;
	}

	/**
	 * invoke the getter method of the specified class. The named method must take
	 * zero formal parameters.
	 * 
	 * <p>Note: If the specified method is found and returns a  <code>boolean</code>, then
	 * an unsuccessful invocation will result in a return value of <code>false</code>.
	 * 
	 * @param clsName - class name
	 * @param methName - method name
	 * @return result of invocation
	 */
	public Object invokeGetter (String clsName, String methName) {
		Class cls = null;
		Object res = null;
		try {
			cls = clCache.classForName(clsName);
			if (cls != null) {
				res = cls.getMethod(methName, (Class[]) null).invoke(null, (Object[]) null);
			}
		}catch (Exception e) {
		}
		return res;
	}

	public Object getDefaultGuiRoot () {
		return null;
	}
	
	/** {@inheritDoc} */
	public ITreeNodeWalker getTreeWalker () {
		if ( treeNodeWalker == null ) {
			treeNodeWalker = new SwtTreeNodeWalker();
			setFilters();
			configuration.setSymbolPool(IConfiguration.ACTF_ID);
			boolean showItems = configuration.getBooleanParameter(IConfiguration.SHOW_ITEMS_KEY);
			if (!showItems){
				SwtNodeItemFilter itemsFilter = new SwtNodeItemFilter();
				treeNodeWalker.addNodeFilter(itemsFilter);
			}
		}
		return treeNodeWalker;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.actf.model.AbstractModel#getNodeLocator()
	 */
	@Override
	public INodeLocator getNodeLocator() {
		if (locator == null){
			locator = new SwtComponentLocator();
		}
		return locator;
	}	
} // SwtGuiModel
