package org.eclipse.actf.model.mozdom;

import java.awt.Rectangle;
import java.lang.reflect.Proxy;
import java.util.HashMap;

import org.eclipse.actf.core.adapt.DefaultAdaptorFactory;
import org.eclipse.actf.core.adapt.IAdaptor;
import org.eclipse.actf.model.event.IModelEventListener;
import org.eclipse.actf.model.event.ModelEventType;
import org.eclipse.actf.model.traverse.ITreeNodeWalker;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.actf.util.resources.ClassLoaderCache;
import org.mozilla.interfaces.nsIDOMCSS2Properties;
import org.mozilla.interfaces.nsIDOMCSSStyleDeclaration;
import org.mozilla.interfaces.nsIDOMDocumentView;
import org.mozilla.interfaces.nsIDOMElement;
import org.mozilla.interfaces.nsIDOMHTMLDocument;
import org.mozilla.interfaces.nsIDOMHTMLElement;
import org.mozilla.interfaces.nsIDOMNode;
import org.mozilla.interfaces.nsIDOMViewCSS;
import org.mozilla.interfaces.nsISupports;
import org.mozilla.xpcom.XPCOMException;

import org.eclipse.actf.model.locate.INodeLocator;
import org.eclipse.actf.model.mozdom.locate.MozillaDomNodeLocator;
import org.eclipse.actf.model.mozdom.traverse.MozillaDomTreeWalker;

/**
 * implementation for the Mozilla DOM browser.
 * 
 * @author Mike Squillace
 *
 */
public class MozillaDomBrowserGuiModel extends DomBrowserGuiModel
{

	public static final String MOZDOM_MODEL = "mozdom";	
	public static final String MOZDOM_ADAPTOR_CLASS = "org.eclipse.actf.accservice.mozdom.adapt.MozillaDomAccessibilityAdaptor";
	
	protected static String[] PACKAGE_LIST = new String[] {
			"java.lang", "org.mozilla.interfaces"};

	private HashMap _tagNameType = new HashMap();

	public MozillaDomBrowserGuiModel () {
		super(MOZDOM_MODEL);
		treeNodeWalker = null;
		Class baseTypeCls = ClassLoaderCache.getDefault().classForName(getBaseType());
		Class adaptorCls = ClassLoaderCache.getDefault().classForName(MOZDOM_ADAPTOR_CLASS);
		IAdaptor adaptor = null;
		try {
			adaptor = (IAdaptor) adaptorCls.newInstance();
		} catch (Exception e) {
			LoggingUtil.println(
					IReporter.SYSTEM_NONFATAL, "Cannot get adaptor class for Mozdom Model", e);
		}
		if (adaptor != null){
			DefaultAdaptorFactory.getInstance().registerAdaptor(nsIDOMHTMLElement.class, adaptor);
		}
	}

	/** {@inheritDoc} */
	public String getDefaultAliasPrefix () {
		return "org.mozilla.interfaces";
	}

	/**
	 * {@inheritDoc}
	 * <p>Includes packages:
	 * <p><ul>
	 * <li> java.lang
	 * <li> org.mozilla.interfaces
	 * </ul>
	 */
	public String[] getPackageNames () {
		return PACKAGE_LIST;
	}

	/** {@inheritDoc} */
	public boolean isVisible (final Object comp) {
		final boolean[] res = new boolean[] {false};
		invokeOnUIThread(new Runnable() {

			public void run () {
				nsIDOMElement element = null;
				if (comp instanceof nsIDOMElement) {
					element = (nsIDOMElement) comp;
				}else if (comp instanceof nsIDOMNode
						&& ((nsIDOMNode) comp).getNodeType() == nsIDOMNode.ELEMENT_NODE) {
					nsIDOMNode node = (nsIDOMNode) comp;
					element = (nsIDOMElement) node.queryInterface(nsIDOMElement.NS_IDOMELEMENT_IID);
				}
				if (element != null) {
					try {
						nsIDOMDocumentView docView = (nsIDOMDocumentView) element.getOwnerDocument().queryInterface(
							nsIDOMDocumentView.NS_IDOMDOCUMENTVIEW_IID);
						nsIDOMViewCSS cssView = (nsIDOMViewCSS) docView.getDefaultView().queryInterface(
							nsIDOMViewCSS.NS_IDOMVIEWCSS_IID);
						nsIDOMCSSStyleDeclaration styles = cssView.getComputedStyle(
							element, "");
						nsIDOMCSS2Properties props = (nsIDOMCSS2Properties) styles.queryInterface(nsIDOMCSS2Properties.NS_IDOMCSS2PROPERTIES_IID);
						res[0] = props.getVisibility().equalsIgnoreCase(
							"visible");
					}catch (Exception e) {
					}
				}
			}
		});
		return res[0];
	}

	/** {@inheritDoc} */
	public boolean requestFocusFor (Object comp) {
		return false;
	}

	/** {@inheritDoc} */
	public boolean isGuiRootType (Class c) {
		return nsIDOMNode.class.isAssignableFrom(c);
	}

	protected String getDomTypesPackage () {
		return "org.mozilla.interfaces";
	}

	protected String getTypeNamePrefix () {
		return "nsIDOMHTML";
	}

	protected String getTypeNameSuffix () {
		return "Element";
	}
	/**
	 * {@inheritDoc}
	 * returns the tag name for the given HTML element (e.g. anchor, input)
	 */
	public String getNodeName (Object element) {
		String nodeName = null, typeName = null;
		
		if (element instanceof nsISupports) {
			try {
				nsIDOMNode node = (nsIDOMNode) ((nsISupports) element).queryInterface(nsIDOMNode.NS_IDOMNODE_IID);
				if (node.getNodeType() == nsIDOMNode.ELEMENT_NODE) {
					nsIDOMHTMLElement htmlElem = (nsIDOMHTMLElement) ((nsISupports) node).queryInterface(nsIDOMHTMLElement.NS_IDOMHTMLELEMENT_IID);
					nodeName = htmlElem.getTagName();
				} else if (node.getNodeType() == nsIDOMNode.DOCUMENT_NODE) {
					nodeName = "document";
				}
			} catch (XPCOMException e) {
			}
		}
		
		Class cls = element.getClass();
		String front = getDomTypesPackage() + "." + getTypeNamePrefix();
		int frontLength = front.length();
		
		if (Proxy.class.isAssignableFrom(cls)) {
			Class[] interfaces = cls.getInterfaces();
			for (Class type : interfaces) {
				if (type.getName().indexOf(front) >= 0) {
					typeName = type.getName();
				}
			}
		}
		
		if (typeName != null && nodeName != null) {
			if (!nodeName.equals("Document")) {
				nodeToTypeNameMap.setProperty(nodeName, typeName);
			} else {
				nodeToTypeNameMap.setProperty("Document", nsIDOMHTMLDocument.class.getName());
			}
		}
		
		return nodeName != null ? nodeName.toLowerCase() : null;
	}

	public String getIIDForTagName (String tagName) {
		String typeName = tagNameToTypeName(tagName);
		String iid = null;
		try {
			Class c = Class.forName(typeName);
			StringBuffer fieldName = new StringBuffer("NS_");
			// skip over '.' and 'ns' in type name
			fieldName.append(typeName.substring(getDomTypesPackage().length() + 1 + 2));
			fieldName.append("_IID");
			iid = (String) c.getField(fieldName.toString().toUpperCase()).get( null);
		}catch (Exception e) {
			LoggingUtil.println(IReporter.SYSTEM_NONFATAL, e);
		}
		return iid;
	}

	/**
	 * get the DOM class/interface type name for the given HTML tag name
	 * cache the infor so we only need to build it once
	 * 
	 * @param tagName HTML 4.01 tag name
	 * @return type name corresponding to given tag name
	 */
	public String tagNameToTypeName (String tagName) {
		String result = null;
		if (tagName != null) {
			result = (String) _tagNameType.get(tagName.toLowerCase());
			if (result == null) {
				result = super.tagNameToTypeName(tagName);
				if (result != null) {
					_tagNameType.put(tagName, result);
				}
			}
		}
		return result;
	}

	protected void initEventIdMap () {
	}
	
	public void unregisterModelEventListener (IModelEventListener listener, ModelEventType[] eventTypes) {
	}
	
	public ModelEventType[] getModelEventTypes (Class c) {
		return null;
	}
	
	public ModelEventType getModelEventType (Object eventId) {
		return null;
	}	
	
	/** {@inheritDoc} */
	public ITreeNodeWalker getTreeWalker () {
		if ( treeNodeWalker == null ) {
			treeNodeWalker = new MozillaDomTreeWalker();
			setFilters();
		}
		return treeNodeWalker;
	}

	public Rectangle getBoundingRectangle(Object element) {
		return null;
	}

	public void highlight(Object element) {
	}

	/* (non-Javadoc)
	 * @see org.eclipse.actf.model.AbstractModel#getNodeLocator()
	 */
	@Override
	public INodeLocator getNodeLocator() {
		if (locator == null){
			locator = new MozillaDomNodeLocator();
		}
		return locator;
	}

	public void registerModelEventListener(IModelEventListener listener, ModelEventType[] eventTypes) {
		registerModelEventListener(listener, eventTypes, null);
	}

	public void registerModelEventListener(IModelEventListener listener, ModelEventType[] eventTypes, Object[] params) {
		// TODO Auto-generated method stub
	}

}
