/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.model.mozdom.traverse;

import java.util.ArrayList;

import org.eclipse.actf.model.DefaultModelFactory;
import org.eclipse.actf.model.InvalidComponentException;
import org.eclipse.actf.model.traverse.AbstractTreeNodeWalker;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.actf.model.mozdom.MozillaDomBrowserGuiModel;
import org.mozilla.interfaces.nsIDOMElement;
import org.mozilla.interfaces.nsIDOMHTMLDocument;
import org.mozilla.interfaces.nsIDOMNode;
import org.mozilla.interfaces.nsIDOMNodeList;
import org.mozilla.xpcom.XPCOMException;


/**
 * a tree walker for the Mozilla browser DOM. This TreeWalker is based on the hierarchy obtained from a 
 * Mozilla browser via XPCOM. Thus, each node in the tree will be expected to be an instance of 
 * <code>org.mozilla.interfaces.nsIDOMNode</code>.
 * 
 * @see <a href="http://www.xulplanet.com/references/xpcomref/">XPCOM API Reference</a>
 * @author Mike Squillace
 *
 */
public class MozillaDomTreeWalker extends AbstractTreeNodeWalker
{

	private MozillaDomBrowserGuiModel _model;

	public MozillaDomTreeWalker () {
		super();
		_model = (MozillaDomBrowserGuiModel) DefaultModelFactory.getInstance().resolveModel(
			MozillaDomBrowserGuiModel.MOZDOM_MODEL);
	}

	/** {@inheritDoc} */
	public Object[] getChildren (Object element)
		throws InvalidComponentException {
		Object[] children = new Object[0];
		
		if (element instanceof nsIDOMNode) {
			Object bridgedChild = getBridgedChild(element);
			nsIDOMNodeList nodeList = ((nsIDOMNode) element).getChildNodes();
			if (children.length == 0 && bridgedChild != null) {
				children = new Object[] {bridgedChild};
			}else {
				ArrayList siblings = new ArrayList();
				long size = nodeList.getLength();
				for (long c = 0; c < size; ++c) {
					nsIDOMNode child = nodeList.item(c);
					String iid = null;
					if (child != null && child.getNodeType() == nsIDOMNode.ELEMENT_NODE) {
						child = (nsIDOMElement) child.queryInterface(nsIDOMElement.NS_IDOMELEMENT_IID);
						String tagName = ((nsIDOMElement) child).getTagName();
						iid = _model.getIIDForTagName(tagName);
						if (iid != null) {
							try {
								child = (nsIDOMElement) child.queryInterface(iid);
							}catch (XPCOMException e) {
								LoggingUtil.println(IReporter.WARNING, "Could not get child for element with tagName: " + tagName);
							}
						}
						if (child != null) {
							siblings.add(child);
						}
					}
				}
				
				children = siblings.toArray();
			}
		}
		
		return children;
	}
	
	public Object[] getStartNodes () {
		// should return all open browser windows?
		return new Object[0];
	}

	/** {@inheritDoc} */
	public Object getParent (Object element) throws InvalidComponentException {
		Object parent = null;
		
		if (element instanceof nsIDOMNode) {
			Object bridgedParent = getBridgedParent(element);
			nsIDOMNode parentNode = ((nsIDOMNode) element).getParentNode();
			if (parentNode == null && bridgedParent != null) {
				parent = bridgedParent;
			} else if (parentNode != null) {
				int nodeType = parentNode.getNodeType();
				if (nodeType == nsIDOMNode.ELEMENT_NODE) {
					parent = (nsIDOMElement) parentNode.queryInterface(nsIDOMElement.NS_IDOMELEMENT_IID);
					String tagName = ((nsIDOMElement) parent).getTagName();
					String iid = _model.getIIDForTagName(tagName);
					if (iid != null) {
						try {
							parent = (nsIDOMElement) parentNode.queryInterface(iid);
						}catch (XPCOMException e) {
						}
					}
				} else if (nodeType == nsIDOMNode.DOCUMENT_NODE) {
					try {
						parent = (nsIDOMHTMLDocument) parentNode.queryInterface(nsIDOMHTMLDocument.NS_IDOMHTMLDOCUMENT_IID);
					}catch (XPCOMException e) {
					}
				}
			}
		}
		
		return parent;
	}
}
