/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  John Buslawski - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.core;

import java.util.HashMap;
import java.util.Map;
import java.util.Observable;

import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.actf.validation.ValidationConstants;
import org.eclipse.actf.validation.rulebase.IRuleBase;
import org.eclipse.actf.validation.validators.IValidationContext;
import org.eclipse.actf.validation.validators.ValidationContextConstants;

/**
 * base implementation of <code>IValidationProcess</code>.
 * 
 * <p>
 * <b>Note</b>: Clients are encouraged to subclass this class rather than to
 * implement <code>IValidationProcess</code> to maintane API compatibility.
 * 
 * @author John Buslawski
 * 
 */
public abstract class AbstractValidationProcess extends Observable implements
		IValidationProcess {

	protected IRuleBase ruleBase;

	protected ExecutionPoint execPoint;

	protected IValidationContext context;

	// the total estimate of work as a number of components
	protected int workEstimate;

	// the current number of components visited
	protected int progress;

	// The notification increment as a percentage. For instance, a value of 10
	// means
	// that a notification is triggered when progress/workEstimate >= 10%, >=
	// 20%, etc.
	// to == 100%
	protected int notificationPercent = 25;

	// notificationIncrement is the calculated increment based on the
	// calculation
	// of workEstimate/notificationPercent.
	protected int notificationIncrement = 0;

	// lastNotification is a marker that indicates the last progress value when
	// a notification
	// was sent to the observers.
	protected int lastNotification = 0;

	// contextCache is a cache of ValicationContext objects that have been
	// created during
	// the validation process. There will be more than one created if the GUI
	// being
	// validated is cross-model. The contexts are keyed by the validator
	// class name.
	protected Map contextCache = new HashMap();

	/** {@inheritDoc} */
	public int getProgress() {
		return progress;
	}

	protected void setProgress(int newProgress) {
		progress = newProgress;
		if (progress > lastNotification + notificationIncrement
				|| progress == workEstimate) {
			setChanged();
			notifyObservers();
			lastNotification = progress;
		}
	}

	/**
	 * increments the progress value by one.
	 * 
	 */
	protected void incrementProgress() {
		progress++;
		if (progress > (lastNotification + notificationIncrement)
				|| progress == workEstimate) {
			setChanged();
			notifyObservers();
			lastNotification = progress;
		}
	}

	protected void resetProgress() {
		progress = 0;
		lastNotification = 0;
		if (workEstimate == 0) {
			notificationIncrement = 0;
		}
	}

	protected void initProgressEstimates() {
		if (workEstimate > 0 && notificationPercent > 0) {
			double percent = notificationPercent / 100.0;
			notificationIncrement = (int) (workEstimate * percent);
		}
	}

	public int getNotificationPercent() {
		return notificationPercent;
	}

	public void setNotificationPercent(int percent) {
		if (percent > 0 && percent <= 100) {
			notificationPercent = percent;
		}
	}

	/** {@inheritDoc} */
	public int getWorkEstimate() {
		return workEstimate;
	}

	/** {@inheritDoc} */
	public IRuleBase getRuleBase() {
		return ruleBase;
	}

	/** {@inheritDoc} */
	public void setRuleBase(IRuleBase iRuleBase) {
		if (iRuleBase != null) {
			ruleBase = iRuleBase;
		} else {
			LoggingUtil.println(IReporter.SYSTEM_NONFATAL,
					"Null IRuleBase in BaseValidationProcess().");
		}
	}

	/** {@inheritDoc} */
	public ExecutionPoint getExecutionPoint() {
		return execPoint;
	}

	/** {@inheritDoc} */
	public void setExecutionPoint(ExecutionPoint executionPoint) {
		if (executionPoint != null) {
			execPoint = executionPoint;
		} else {
			LoggingUtil.println(IReporter.SYSTEM_NONFATAL,
					"Null ExecutionPoint in BaseValidationProcess().");
		}
	}

	/** {@inheritDoc} */
	public void setValidationContext(IValidationContext valContext) {
		if (valContext != null) {
			context = valContext;
		} else {
			LoggingUtil.println(IReporter.SYSTEM_NONFATAL,
					"Null IValidationContext in BaseValidationProcess().");
		}
	}

	/** {@inheritDoc} */
	public IValidationContext getValidationContext() {
		return context;
	}

	protected synchronized boolean isTerminated() {
		IConfiguration config = (IConfiguration) context
				.getParameter(ValidationContextConstants.CONFIGURATION);
		config.setSymbolPool(ValidationConstants.VALIDATION_ID);
		return config.getBooleanParameter(ValidationConstants.VALIDATION_TERMINATED_KEY);
	}
}
