/*******************************************************************************
 * Copyright (c) 2004, 2007 IBM Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  John Buslawski - initial API and implementation
 *******************************************************************************/

package org.eclipse.actf.validation.rulebase;

import java.net.URL;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Pattern;

import org.eclipse.actf.core.config.ConfigurationException;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.actf.util.resources.IResourceLocator;

/**
 * class that manages the ACTF RuleBases. The RuleBases are stored in a Map
 * using their name as a key. The RuleBaseManager can also find a IRuleBase for a
 * target class by it's fully qualified class name.
 * 
 * <p>
 * The RuleBases are loaded as needed. When a target is to be validated, it's
 * fully qualified class name can be used to match a regular expression defining
 * which IRuleBase is to be used.
 * 
 * <p><b>Note</b>: Only document rulebases (i.e. rulebases defined with validation documents) are currently supported.
 * 
 * @author John Buslawski
 * 
 */
public class RuleBaseManager 
{

	private IRuntimeContext context;
	private IResourceLocator resourceLocator;
	private IConfiguration config;

	// a handle to the singleton ReflectionDocumentRuleFactory
	private static RuleBaseManager rbmInstance;

	// ruleBaseInstanceMap is a map of IRuleBase name/IRuleBase instances
	private Map ruleBaseInstanceMap = new HashMap();

	// targetToRuleBaseMap is a map of target names/List of IRuleBase names
	private Map targetToRuleBaseMap = new HashMap();

	// ruleBaseDescriptorMap is a map of IRuleBase names/RuleBaseDescriptors 
	private Map ruleBaseDescriptorMap = new HashMap();

	/**
	 * @return The unique instance of this class.
	 */
	public static RuleBaseManager getInstance() {
		if (null == rbmInstance) {
			rbmInstance = new RuleBaseManager();
		}
		return rbmInstance;
	}

	protected RuleBaseManager() {
		try {
			context = RuntimeContextFactory.getInstance().getRuntimeContext();
			resourceLocator = context.getResourceLocator();
			config = context.getConfiguration();
		} catch (ConfigurationException e) {
			// Could not obtain a Configuration. We can't do much here but
			// quit.
			throw new RuntimeException(e.getMessage(), e);
		}
	}

	public void addRuleBase(IRuleBase iRuleBase) {
		if (iRuleBase != null) {
			String name = iRuleBase.getName();
			if (name != null && name.length() > 0) {
				ruleBaseInstanceMap.put(name, iRuleBase);
			}
		}
	}

	public void clearAll() {
		clearRuleBases();
		clearRuleBaseMappings();
		clearTargetToRuleBaseLinks();
	}

	public void clearRuleBases() {
		ruleBaseInstanceMap.clear();
	}

	public void clearRuleBaseMappings() {
		ruleBaseDescriptorMap.clear();
	}

	public void clearTargetToRuleBaseLinks() {
		targetToRuleBaseMap.clear();
	}

	public IRuleBase getRuleBase(String name) {
		if (!ruleBaseInstanceMap.containsKey(name)) {
			DocumentRuleBaseDescriptor descriptor = (DocumentRuleBaseDescriptor) ruleBaseDescriptorMap.get(name);
			if (descriptor != null) {
				loadRuleBase(descriptor);
			}
		}
		return (IRuleBase) ruleBaseInstanceMap.get(name);
	}
/* MAS: disable for now as cannot take advantage of lazy-loading of rbs */
//	public IRuleBase[] getRuleBases() {
//		return (IRuleBase[]) ruleBaseInstanceMap.entrySet().toArray(new IRuleBase[ruleBaseInstanceMap.size()]);
//	}

	public void removeRuleBase(String name) {
		
		if (name != null && ruleBaseInstanceMap.containsKey(name)) {ruleBaseInstanceMap.remove(name);
		
		}
	}

	public IRuleBaseDescriptor[] getRuleBaseDescriptors() {
		return (IRuleBaseDescriptor[]) ruleBaseDescriptorMap.values().toArray(new IRuleBaseDescriptor[ruleBaseDescriptorMap.size()]);
	}
	
	public void addRuleBase(IRuleBaseDescriptor descriptor) {
		String rbName = descriptor.getRuleBaseName();
		ruleBaseDescriptorMap.put(rbName, descriptor);

		// Now link the targets to the rule base names.
		String[] targets = ((DocumentRuleBaseDescriptor) descriptor).getTargets();
		for (int i = 0; i < targets.length; i++) {
			linkTargetToRuleBase(targets[i], rbName);
		}
	}

	public void linkTargetToRuleBase(String targetName, String ruleBaseName) {
		// TODO not sure if we should allow targets to be linked to RuleBases
		// that haven't
		// been defined yet. For now we will.
		// For now, we'll assume that only regular expressions are put into the
		// map
		// that associates target names to RuleBases.
		Set ruleBaseSet;
		boolean patternFound = false;
		Iterator iter = targetToRuleBaseMap.keySet().iterator();
		while (iter.hasNext()) {
			Object regexpPattern = iter.next();
			if (regexpPattern instanceof Pattern) {
				if (((Pattern) regexpPattern).matcher(targetName).matches()) {
					patternFound = true;
					ruleBaseSet = (Set) targetToRuleBaseMap.get(regexpPattern);
					if (!ruleBaseSet.contains(ruleBaseName)) {
						ruleBaseSet.add(ruleBaseName);
						targetToRuleBaseMap.put(regexpPattern, ruleBaseSet);
						break;
					}
				} else if (((Pattern) regexpPattern).pattern().equals(
						targetName)) {
					patternFound = true;
					ruleBaseSet = (Set) targetToRuleBaseMap.get(regexpPattern);
					if (!ruleBaseSet.contains(ruleBaseName)) {
						ruleBaseSet.add(ruleBaseName);
						targetToRuleBaseMap.put(regexpPattern, ruleBaseSet);
						break;
					}
				}
			}
		}
		// If the pattern wasn't found then this must be the first time that
		// we've seen it. Put it into the map with a new Set containing the
		// IRuleBase name.
		if (!patternFound) {
			ruleBaseSet = new HashSet();
			ruleBaseSet.add(ruleBaseName);
			targetToRuleBaseMap.put(Pattern.compile(targetName), ruleBaseSet);
		}
	}

	public void unlinkTargetFromRuleBase(String targetName, String ruleBaseName) {
		Set ruleBaseSet;
		boolean patternFound = false;
		Iterator iter = targetToRuleBaseMap.keySet().iterator();
		while (iter.hasNext()) {
			Object regexpPattern = iter.next();
			if (regexpPattern instanceof Pattern) {
				if (((Pattern) regexpPattern).matcher(targetName).matches()) {
					patternFound = true;
					ruleBaseSet = (Set) targetToRuleBaseMap.get(regexpPattern);
					if (!ruleBaseSet.contains(ruleBaseName)) {
						ruleBaseSet.remove(ruleBaseName);
						targetToRuleBaseMap.put(regexpPattern, ruleBaseSet);
						break;
					}
				}
			}
		}
		if (!patternFound) {
			LoggingUtil.println(LoggingUtil.PROCESS_ALL,
					"RuleBaseManager.unlinkTargetToRuleBase() - target name "
							+ targetName + " was not defined in map.");
		}
	}

	public void removeLinksToTarget(String targetName) {
		if (targetToRuleBaseMap.containsKey(targetName)) {
			targetToRuleBaseMap.remove(targetName);
		}
	}

	public IRuleBase[] getRuleBasesByTarget(Object target) {
		String name = null;
		Set ruleBaseNameSet = null;
		if (target != null) {
			// find all classes and interfaces of which target is an instance
			List classes = new LinkedList();
			Class cls = target.getClass();
			classes.addAll(Arrays.asList(cls.getInterfaces()));
			classes.add(cls);
			while ((cls = cls.getSuperclass()) != null) {
				classes.add(cls);
			}
			
			// for each type of which target is an instance, attempt to retrieve
			// a rulebase set for the given type name
			for (Iterator citer = classes.iterator(); citer.hasNext() & ruleBaseNameSet == null;) {
				name = ((Class) citer.next()).getName();
				ruleBaseNameSet = (Set) targetToRuleBaseMap.get(name);
				
				// if no set is cached or none is yet found, iterate through registered rulebase targets
				// (which are currently assumed to be regular expressions gathered from rulebases.xml)
				// and test whether or not type matches regexp; if a match is found, the name itself of the type
				// replaces the regexp pattern as the key into the map
				for (Iterator iter = targetToRuleBaseMap.keySet().iterator(); iter.hasNext() & ruleBaseNameSet == null;) {
					Object pattern = iter.next();
					if (pattern instanceof Pattern
							&& ((Pattern) pattern).matcher(name).matches()) {
						ruleBaseNameSet = (Set) targetToRuleBaseMap.get(pattern);
						targetToRuleBaseMap.put(name, ruleBaseNameSet);
					}
				}
			}
		}
		
		// ruleBaseSet contains the names of the RuleBases linked to the target.
		// get the actual RuleBases and return them.
		if (ruleBaseNameSet != null && !ruleBaseNameSet.isEmpty() ) {
			LinkedList ruleBases = new LinkedList();
			Iterator setIter = ruleBaseNameSet.iterator();
			while (setIter.hasNext()) {
				String ruleBaseName = (String) setIter.next();
				IRuleBase iRuleBase = getRuleBase(ruleBaseName);
				if (iRuleBase != null && iRuleBase.isEnabled()) {
					ruleBases.add(iRuleBase);
				} else {
					LoggingUtil.println(IReporter.SYSTEM_NONFATAL,
						"RuleBaseManager.getRuleBasesByTarget() - rule base name " + ruleBaseName + " does not have a corresponding instance or is disabled.");
				}
			}
			
			return (IRuleBase[]) ruleBases.toArray(new IRuleBase[ruleBases.size()]);
		} else {
			LoggingUtil.println(LoggingUtil.PROCESS_ALL,
					"RuleBaseManager.getRuleBasesByTarget() - target name "
							+ name + " was not defined in map.");
			return new IRuleBase[0];
		}
	}

	public IRuleBase[] getRuleBasesByTargetName(String targetName) {
		Set ruleBaseNameSet = null;
		Iterator iter = targetToRuleBaseMap.keySet().iterator();
		while (iter.hasNext()) {
			Object regexpPattern = iter.next();
			if (regexpPattern instanceof Pattern) {
				if (((Pattern) regexpPattern).matcher(targetName).matches()) {
					ruleBaseNameSet = (Set) targetToRuleBaseMap.get(regexpPattern);
					// Iterate through the RuleBases and load them if necessary.
					// The loadOnDemand
					// value doesn't matter in this case since we want to load
					// them before we
					// actually use them.
					Iterator rbIter = ruleBaseNameSet.iterator();
					while (rbIter.hasNext()) {
						String rbName = (String) rbIter.next();
						if (!ruleBaseInstanceMap.containsKey(rbName)
								&& ruleBaseDescriptorMap.containsKey(rbName)) {
							loadRuleBase((DocumentRuleBaseDescriptor) ruleBaseDescriptorMap.get(rbName));
						}
					}
					break;
				}
			}
		}
		if (ruleBaseNameSet != null && ruleBaseNameSet.size() > 0) {
			LinkedList ruleBases = new LinkedList();
			Iterator setIter = ruleBaseNameSet.iterator();
			while (setIter.hasNext()) {
				String ruleBaseName = (String) setIter.next();
				if (ruleBaseInstanceMap.containsKey(ruleBaseName)) {
					ruleBases.add(ruleBaseInstanceMap.get(ruleBaseName));
				} else {
					LoggingUtil.println(LoggingUtil.PROCESS_ALL,
						"RuleBaseManager.getRuleBasesByTarget() - rule base name " + ruleBaseName + " does not have a corresponding instance.");
				}
			}
			return (IRuleBase[]) ruleBases.toArray(new IRuleBase[ruleBases.size()]);
		} else {
			LoggingUtil.println(LoggingUtil.PROCESS_ALL,
					"RuleBaseManager.getRuleBasesByTargetName() - target name "
							+ targetName + " was not defined in map.");
			return new IRuleBase[0];
		}
	}

	protected void loadRuleBase(DocumentRuleBaseDescriptor rbDescriptor) {
		String rbName = rbDescriptor.getRuleBaseName();
		String sourceType = rbDescriptor.getType();
		String rbFactory = rbDescriptor.getRuleBaseFactory();
		if (sourceType.equals(DocumentRuleBaseDescriptor.DOCUMENT)) {
			String sourceLocation = rbDescriptor.getSourceLocation();
			try {
				IRuleBaseFactory rbf = (IRuleBaseFactory) Class.forName(rbFactory).newInstance();
				if (rbf != null) {
					URL docUrl = resourceLocator.getResource(sourceLocation);
					IRuleBase rb = rbf.buildRuleBase(docUrl);
					if (rb != null) {
						addRuleBase(rb);
					}
				} else {
					LoggingUtil.println(IReporter.ERROR, "Unable to create rulebaseFactory for IRuleBase : " + rbName );
				}
			} catch (Exception e) {
				LoggingUtil.println(IReporter.SYSTEM_FATAL, "Error instantiating rulebase " + rbName, e);
			}
		} else {
			throw new UnsupportedOperationException("Cannot load IRuleBase  " + rbName + " of type " + sourceType);
		}
	}
	
	public void buildRuleBases(RuleBaseMapDocument rbDoc) {
		IRuleBaseDescriptor[] descriptors = rbDoc.getRuleBaseDescriptors(); 
		for (int i = 0; i < descriptors.length; ++i) {
			addRuleBase(descriptors[i]);
		}
	}
	
}
