package org.eclipse.actf.validation.rules;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.actf.core.adapt.DefaultAdaptorFactory;
import org.eclipse.actf.core.adapt.IAdaptor;
import org.eclipse.actf.core.adapt.IAdaptorFactory;
import org.eclipse.actf.model.IGuiModel;
import org.eclipse.actf.model.IModel;
import org.eclipse.actf.model.InvalidComponentException;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.actf.validation.core.Parameter;
import org.eclipse.actf.validation.core.Reflector;
import org.eclipse.actf.validation.valdoc.codeproc.ICodeProcessor;
import org.eclipse.actf.validation.validators.IValidationContext;
import org.eclipse.actf.validation.validators.ValidationContextConstants;
import org.w3c.dom.Attr;

/**
 * @author John Buslawski
 *
 */
public class ReflexiveCriterion extends AbstractRule
{

	// TODO Refactor notes
	// In release 2.0, the difference between a rule and criterion is:
	// 1) there is only one criterion per top level rule,
	// 2) criterion are not nested
	// 3) criterion never return a real validation record. When a criterion is evaluated
	//		it is either true or false.
	// In future releases, the IRule implementation needs to be refactored so that
	// the IRule interface can be shared between Rules and Criterion.

	private Reflector reflector = new Reflector();
	private String[] methodNames = null;
	private Class[] srcClasses = null;
	private Object[] srcObjects = null;
	private String targetValue = null;

	public void initialize (Object content) throws Exception {
		if (content instanceof Attr) {
			Attr attr = (Attr) content;
			buildReflectionParameters(attr);
			targetValue = getAliasValue(attr.getValue());
		}
	}

	public boolean apply (IValidationContext context, Object target) throws Exception {
		// execute the presence test
		boolean result = super.apply(context, target);
		// if not enabled, presence test failed,
		// or ignoring null target, then return
		if (isEnabled() && result && !isIgnoringNullTarget(target)) {
			buildReflectionTargets(context, target);
			result = invokeMethod(context, target);
		}
		
		return result;
	}

	private void buildReflectionParameters (Attr attr) {
		methodNames = new String[] {
			attr.getLocalName(),
			"get" + LoggingUtil.firstCharToUpper(attr.getLocalName())
		};
	}

	/**
	 * 
	 * Use the IAdaptorFactory for this model to find other candidate classes
	 * for this component when using reflection to find a method. This technique 
	 * uses the Eclipse IAdaptable pattern.
	 *
	 * @param component to evaluate 
	 */
	private void buildReflectionTargets (IValidationContext context, Object component) {
		// The default list of classes and objects contain the component
		// itself at a minimum. 
		srcClasses = new Class[] {component.getClass()};
		srcObjects = new Object[] {component};
		// Find the IAdaptorFactory for this model and instantiate it.
		// The IAdaptorFactory class can be found in the Configuration object.
		IModel model = (IModel) context.getParameter(ValidationContextConstants.MODEL);
		IAdaptorFactory adaptorFactory = DefaultAdaptorFactory.getInstance(); 
		if (adaptorFactory != null) {
			// Create some temporary collections and seed them with the 
			// target class types and target object.
			List classList = new ArrayList();
			classList.add(component.getClass());
			List objectList = new ArrayList();
			objectList.add(component);
			// get the possible Adaptors for this type
			IAdaptor[] adaptors = adaptorFactory.getAllAdaptors(component.getClass());
			Class[] classTypes = null;
			if (adaptors.length > 0) {
				for (int i = 0; i < adaptors.length; i++) {
					classTypes = adaptors[i].getSupportedTypes();
					// Iterate through the supported class types and try to adapt the target
					// to the class type. If it can be adapted then put it on the list of
					// reflection candidates.
					for (int j = 0; j < classTypes.length; j++) {
						Object adaptedObject = null;
						try {
							adaptedObject = adaptors[i].adapt(
								component, classTypes[j]);
						}catch (Exception e) {
							LoggingUtil.println( IReporter.SYSTEM_NONFATAL, "Could not adapt " + component + " to type " + classTypes[j].getName(), e);
						}
						if (adaptedObject != null) {
							LoggingUtil.println(LoggingUtil.PROCESS_ALL,
								"ReflexiveRule.buildReflectionTargets(Object), adding class type: " + classTypes[j].getName() + " to list of reflection targets.");
							classList.add(classTypes[j]);
							objectList.add(adaptedObject);
						}
					}
				}
			}
			srcClasses = (Class[]) classList.toArray(new Class[classList.size()]);
			srcObjects = objectList.toArray(new Object[objectList.size()]);
		}
	}

	private boolean invokeMethod (IValidationContext context, Object component)
		throws InvalidComponentException {
		final Reflector.Result[] methResult = new Reflector.Result[1];
		final Throwable[] throwable = new Throwable[1];
		final Object[] result = new Object[1];
		IModel model = (IModel) context.getParameter(ValidationContextConstants.MODEL);
		ICodeProcessor codeProcessor = (ICodeProcessor) context.getParameter(ValidationContextConstants.CODE_PROCESSOR);
		Parameter[] params = null;
		String param = getAttribute(RuleConstants.PARAM);
		Object propValue = null;
		int mn = 0; // index for method names array
		boolean passed = true;
		
		// algorithm for finding getter method to verify property:
		// call method with name of element on control
		// call method "get" + firstCharToUpper(name of element) on control
		// call method with name of element on default context
		// call method "get" + firstCharToUpper(name of element) on default
		// context
		// Note: provision for one param given as 'rib:param' in xml doc
		CLASS_LOOP: for (int cl = 0; cl < srcClasses.length; ++cl) {
			for (mn = 0; mn < methodNames.length; ++mn) {
				try {
					params = new Parameter[0];
					methResult[0] = reflector.findMethod(
						srcObjects[cl], methodNames[mn], params);
					if (methResult[0] != null) {
						final int clx = cl;
						Runnable runnable = new Runnable() {
							public void run () {
								try {
									result[0] = methResult[0].invoke(srcObjects[clx]);
								}catch (Throwable e) {
									throwable[0] = e;
								}
							}
						};
						
						if (model instanceof IGuiModel) {
							((IGuiModel) model).invokeOnUIThread(runnable);
						} else {
							runnable.run();
						}
						
						if (throwable[0] != null) {
							Throwable t = throwable[0];
							if (t instanceof InvalidComponentException) {
								throw (InvalidComponentException) t;
							}else {
								throw new RuntimeException(t.getMessage());
							}
						}
						propValue = result[0];
						LoggingUtil.println(LoggingUtil.ALL, "Successful invokation of method"
									+ methResult[0].getMember() + " returned value: " + propValue);
						if (propValue != null) {
							LoggingUtil.println(LoggingUtil.ALL, "Breaking with value "
										+ propValue + " for property " + getName());
							break CLASS_LOOP;
						}else {
							LoggingUtil.println(LoggingUtil.PROCESS_ALL,
								"No value returned for property/method " + getName() + (param != null ? " and parameter " + param : ""));
							break CLASS_LOOP;
						}
					}else {
						// attempt to treat property as referring to a public
						// field
						propValue = reflector.getFieldValue(
							methodNames[mn], srcObjects[cl]);
						if (propValue != null) {
							LoggingUtil.println(LoggingUtil.ALL, "Breaking with field value " + propValue);
							break CLASS_LOOP;
						}
					}
				}catch (InvalidComponentException e) {
					throw e;
				}catch (Exception e) {
					LoggingUtil.println(IReporter.SYSTEM_NONFATAL,
						"Exception caught while attempting to locate - " + srcObjects[cl] + "." + methodNames[mn], e);
					continue;
				}
			}
		}
		
		// now that we have the result of the getter, we are ready to
		// validate the property
		if (propValue != null) {
			if (!targetValue.equals(propValue)) {
				passed = false;
			}
		} else {
			// not leaf but propValue == null
			passed = false;
		}
		
		return passed;
	}
}