/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*  Kentarou FUKUDA - [232278]
*******************************************************************************/ 

package org.eclipse.actf.validation.ui;

import java.io.File;
import java.net.URL;
import java.util.HashSet;
import java.util.IdentityHashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.StringTokenizer;

import org.eclipse.actf.core.config.ConfigurationException;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.util.logging.AbstractReporter;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.resources.ClassLoaderCache;
import org.eclipse.actf.util.resources.EclipseResourceLocator;
import org.eclipse.actf.validation.ValidationConstants;
import org.eclipse.actf.validation.ui.preferences.ValidationPreferenceInitializer;
import org.eclipse.actf.validation.ui.preferences.ValidationPreferencePage;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Preferences;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;

/**
 * The main plugin class to be used in the desktop.
 */
abstract public class AbstractValidationUIPlugin extends AbstractUIPlugin {

	abstract public String getPluginId();

	abstract protected String getTraceOptionId();

	abstract protected String getDebugOptionId();

	abstract protected String getLogOptionId();
	
	abstract protected String getDefaultImageName();
	
	abstract protected String getValidationTraceFile();
	
	protected ImageRegistry imageRegistry;
	protected Set imageKeys = new HashSet();
	protected Set colors = new HashSet();
	protected IRuntimeContext runtimeContext;
	protected ClassLoaderCache clCache = ClassLoaderCache.getDefault();

	public void initializeImageRegistry(ImageRegistry imageRegistry) {
		ImageDescriptor id = getImageDescriptor("icons/logo");
		if (id != null) {
			registerImage(getDefaultImageName(), id);
		}
	}

	public ImageDescriptor getImageDescriptor(String imageName) {
		ImageDescriptor id = null;
		int posn = imageName.indexOf('/'); // only look in name
		String xname = posn >= 0 ? imageName.substring(posn + 1) : imageName;
		posn = xname.indexOf('.'); // add default extension
		if (posn < 0) {
			imageName += ".gif";
		}
		IPath path = new Path(imageName);
		URL url = this.find(path);
		if (url == null) {
			Bundle bundle = Platform.getBundle(getPluginId());
			if (bundle != null) {
				url = Platform.find(bundle, path);
			}
		}
		if (url != null) {
			id = ImageDescriptor.createFromURL(url);
		}
		return id;
	}

	public void registerImage(String key, String imageName) {
		/*
		 * //ImageDescriptor id =
		 * ImageDescriptor.createFromFile(this.getClass(), imageName); Path path =
		 * new Path(imageName); File file = path.toFile(); try { String name =
		 * file.getCanonicalPath(); ImageDescriptor id =
		 * ImageDescriptor.createFromFile(null, name); trace("registerImage " +
		 * key + "," + imageName + ": " + id); registerImage(key, id); } catch
		 * (IOException e) { e.printStackTrace(); }
		 */
		ImageDescriptor id = getImageDescriptor(imageName);
		registerImage(key, id);
	}

	public void registerImage(String key, ImageDescriptor image) {
		try {
			imageRegistry.put(key, image);
			imageKeys.add(key);
		} catch (IllegalArgumentException e) { // dup
			// e.printStackTrace();
		}
	}

	/** name format is "key:imageName" */
	public void registerImages(String[] imageNames) {
		for (int i = 0; i < imageNames.length; i++) {
			StringTokenizer st = new StringTokenizer(imageNames[i], ":");
			String key = st.nextToken();
			String name = st.nextToken();
			registerImage(key, name);
		}
	}

	public void registerImage(String key, Image image) {
		try {
			imageRegistry.put(key, image);
			imageKeys.add(key);
		} catch (IllegalArgumentException e) { // dup
			// e.printStackTrace();
		}
	}

	public Image getRegisteredImage(String key) {
		Image i = imageRegistry.get(key);
		if (i == null) {
			i = imageRegistry.get(getDefaultImageName());
		}
		// trace("getRegisteredImage " + key + ": " + i);
		return i;
	}

	protected IConfiguration configuration;

	/**
	 * The constructor.
	 */
	public AbstractValidationUIPlugin() {
		super();
	}

	protected boolean useNormalColors;

	public boolean useNormalBackgroundColor() {
		return useNormalColors;
	}

	protected boolean resetColors;

	/**
	 * This method is called upon plug-in activation
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		
		runtimeContext = RuntimeContextFactory.getInstance().getRuntimeContext();
		EclipseResourceLocator locator = (EclipseResourceLocator) runtimeContext.getResourceLocator();

		// provide a way for retrieving classes and resources from all bundles
		locator.registerBundleName(getPluginId());
		clCache.put(getPluginId(), getClass().getClassLoader());
		
		// should not need to do this to force preference loading
		// AKF turns out this is useless for both Java and Mozdom - I guess they are not on the classpath at the time this is executed
		// it doesn't find either one
		String[] pluginNames = new String[] {
				"org.eclipse.actf.model.javapp.JavaModelPlugin",
				"org.eclipse.actf.validation.ValidationCorePlugin",
				"org.eclipse.actf.accservice.core.AccessibilityServicePlugin",
				"org.eclipse.actf.model.mozdom.MozdomModelPlugin"
		};
		
		for (String pluginName : pluginNames) {
			Class pluginCls = ClassLoaderCache.getDefault().classForName(pluginName);
			if (pluginCls != null) {
				pluginCls.getMethod("getDefault", new Class[0]).invoke(null, (Object[]) null);
			}
		}
		
		imageRegistry = super.getImageRegistry(); // local cache
		try {
			IRuntimeContext runtimeContext = RuntimeContextFactory.getInstance().getRuntimeContext();
			configuration = runtimeContext.getConfiguration();
		} catch (ConfigurationException e) {
			// Could not obtain a Configuration. We can't do much here but
			// quit.
			throw new CoreException(new Status(IStatus.ERROR, getPluginId(),
					IStatus.OK, e.getMessage(), e));
		}
		
		initializeUserPreferences();
		prepareTraceFacility();
		IPreferenceStore store = getPreferenceStore();
		store.addPropertyChangeListener(new IPropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				String property = e.getProperty();
				if (property.equals(ValidationPreferenceInitializer.P_COLORS_NORMAL)) {
					Object o = e.getNewValue();
					if (o instanceof Boolean) {
						useNormalColors = ((Boolean) e.getNewValue())
								.booleanValue();
					} else if (o instanceof String) {
						useNormalColors = ((String) e.getNewValue())
								.equals("true");
					}
					resetColors = true; // forces reload next time color used
				} else if (property
						.equals(ValidationPreferenceInitializer.P_COLOR_ERROR)
						|| property
								.equals(ValidationPreferenceInitializer.P_COLOR_WARNING)
						|| property
								.equals(ValidationPreferenceInitializer.P_COLOR_INFO)) {
					resetColors = true; // forces reload next time color used
				} else {
					setActfConfig(property, e.getNewValue());
				}
			}
		});
	}

	protected IReporter tracer;
	protected IReporter getTracer() {
		return tracer;
	}

	protected String traceStream;

	protected int traceLevel = IReporter.WARNING;

	protected void prepareTraceFacility() {
		String trace = Platform.getDebugOption(getTraceOptionId());
		if (isDebugging(getDebugOptionId()) && trace != null
				&& trace.equalsIgnoreCase("true")) {
			String levelStr = Platform.getDebugOption(getTraceOptionId() + "/level");
			traceStream = Platform.getDebugOption(getTraceOptionId() + "/stream");
			if (traceStream == null) {
				traceStream = setupDefaultTraceStream();
				if (traceStream == null) {
					traceStream = "stdout";
				}
			}
			if (levelStr == null) {
				levelStr = new Integer(IReporter.WARNING).toString();
			}
			try {
				traceLevel = Integer.parseInt(levelStr);
			} catch (NumberFormatException e) {
				traceLevel = IReporter.WARNING;
			}
			tracer = AbstractReporter.getReporter(traceLevel, traceStream);
			tracer.setSourceID(IReporter.TRACE);
		}
	}

	/**
	 * This method is called when the plug-in is stopped
	 */
	public void stop(BundleContext context) throws Exception {
		// clear images
		for (Iterator i = imageKeys.iterator(); i.hasNext();) {
			String key = (String) i.next();
			imageRegistry.remove(key);
		}
		// clear colors
		for (Iterator i = colors.iterator(); i.hasNext();) {
			Color color = (Color) i.next();
			if (!color.isDisposed()) {
				color.dispose();
			}
			i.remove();
		}
		// disposeErrorColor();
		// disposeWarningColor();
		// disposeOkColor();
		super.stop(context);
	}

	/*
	 * protected void disposeErrorColor() { if (errorBgColor != null &&
	 * !errorBgColor.isDisposed()) { errorBgColor.dispose(); errorBgColor =
	 * null; } } protected void disposeWarningColor() { if (warningBgColor !=
	 * null && !warningBgColor.isDisposed()) { warningBgColor.dispose();
	 * warningBgColor = null; } } protected void disposeOkColor() { if
	 * (okBgColor != null && !okBgColor.isDisposed()) { okBgColor.dispose();
	 * okBgColor = null; } }
	 */
	protected IdentityHashMap creationContexts = new IdentityHashMap();

	/**
	 * @return Returns the creationContexts.
	 */
	public IdentityHashMap getCreationContexts() {
		return creationContexts;
	}

	public StackTraceElement[] getCreationContext(Widget w) {
		return (StackTraceElement[]) creationContexts.get(w);
	}

	/**
	 * @param creationContexts
	 *            The creationContexts to set.
	 */
	public void setCreationContexts(IdentityHashMap creationContexts) {
		this.creationContexts = creationContexts;
	}

	public void setCreationContext(Widget w, StackTraceElement[] trace) {
		if (creationContexts.containsKey(w) && trace == null) {
			creationContexts.remove(w);
		} else {
			creationContexts.put(w, trace);
		}
	}

	protected Color errorBgColor, warningBgColor, infoBgColor, markerColor;

	public Color createColor(Control ctl, String key) {
		RGB rgb = PreferenceConverter.getColor(getPreferenceStore(), key);
		Color c = new Color(ctl.getDisplay(), rgb);
		colors.add(c);
		return c;
	}

	protected void resetColors() {
		if (resetColors) {
			resetColors = false;
			// disposeErrorColor();
			// disposeWarningColor();
			// disposeOkColor();
			errorBgColor = null;
			warningBgColor = null;
			infoBgColor = null;
		}
	}

	protected void validateColor(Color c) {
		if (c == null || c.isDisposed()) {
			throw new IllegalStateException("color is null or disposed");
		}
	}

	public Color getErrorBackgroundColor(Control ctl) {
		resetColors();
		Color c = errorBgColor;
		if (useNormalColors) {
			c = ctl.getBackground();
		} else {
			if (c == null) {
				c = createColor(ctl, ValidationPreferenceInitializer.P_COLOR_ERROR);
				errorBgColor = c;
			}
		}
		validateColor(c);
		return c;
	}

	public Color getWarningBackgroundColor(Control ctl) {
		resetColors();
		Color c = warningBgColor;
		if (useNormalColors) {
			c = ctl.getBackground();
		} else {
			if (c == null) {
				c = createColor(ctl, ValidationPreferenceInitializer.P_COLOR_WARNING);
				warningBgColor = c;
			}
		}
		validateColor(c);
		return c;
	}

	public Color getInfoBackgroundColor(Control ctl) {
		resetColors();
		Color c = infoBgColor;
		if (useNormalColors) {
			c = ctl.getBackground();
		} else {
			if (c == null) {
				c = createColor(ctl, ValidationPreferenceInitializer.P_COLOR_INFO);
				infoBgColor = c;
			}
		}
		validateColor(c);
		return c;
	}

	public Color getNormalBackgroundColor(Control ctl) {
		return ctl.getBackground();
	}

	public Color getNormalForegroundColor(Control ctl) {
		return ctl.getForeground();
	}

	protected String setupDefaultTraceStream() {
		boolean success = true;
		String traceStream = null;
		String tmpdir = System.getProperty("java.io.tmpdir") + "actf";
		File actfTmpDir = new File(tmpdir);
		if (!actfTmpDir.exists()) {
			try {
				success = actfTmpDir.mkdir();
			} catch (Exception e) {
				// TODO need to log exception
				success = false;
			}
		}
		if (success) {
			traceStream = System.getProperty("java.io.tmpdir") + "actf"
					+ File.separator + getValidationTraceFile();
		} else {
			traceStream = System.getProperty("java.io.tmpdir")
					+ getValidationTraceFile();
		}
		return traceStream;
	}

	public void log(String option, int sev, int code, String message,
			Throwable t) {
		if (message == null) {
			message = "<no message>";
		}
		if (isDebugging(option)) {
			Status status = new Status(sev, getPluginId(), code, message, t);
			getLog().log(status);
		}
	}

	public void logInfo(String message) {
		log(getLogOptionId(), IStatus.INFO, 0, message, (Throwable) null);
	}

	public void logError(String message) {
		log(getLogOptionId(), IStatus.ERROR, 0, message, (Throwable) null);
	}

	public void logException(String message, Throwable t) {
		String tname = t.getClass().getName();
		String msg = t.getMessage();
		msg = msg != null && msg.length() > 0 ? tname + " - " + msg : tname;
		Throwable ct = t.getCause();
		if (ct != null) {
			msg += " with base cause: " + ct.getMessage();
		}
		log(getLogOptionId(), IStatus.ERROR, 1, message != null ? (message
				+ " caused by " + msg) : t.getMessage(), t);
	}

	public void logException(Throwable t) {
		String msg = t.getMessage();
		logException(msg != null && msg.length() > 0 ? msg : "<no message>", t);
	}

	public boolean isDebugging(String option) {
		String value = Platform.getDebugOption(option);
		return super.isDebugging() && value != null
				&& "true".equalsIgnoreCase(value);
	}

	public void trace(String option, String message) {
		if (isDebugging(option)) {
			getTracer().report(IReporter.INFO, message);
		}
	}

	public void trace(String message) {
		trace(getTraceOptionId(), message);
	}

	public void trace(int level, String msg) {
		if (isDebugging(getTraceOptionId())) {
			getTracer().report(level, msg);
		}
	}

	public void trace(Object source, String message) {
		trace(getTraceOptionId(), source.getClass().getName() + ": " + message);
	}

	public void trace(String message, Throwable t) {
		if (isDebugging(getTraceOptionId())) {
			getTracer().report(IReporter.ERROR, message, t);
		}
	}

	public void trace(Throwable t) {
		String msg = t != null ? t.getMessage() : "<no message>";
		trace(msg != null && msg.length() > 0 ? msg : "<no message>", t);
	}

	private void initializeUserPreferences() {
		Preferences prefs = getPluginPreferences();
		String[] keys = prefs.defaultPropertyNames();
		for (int i = 0; i < keys.length; i++) {
			String property = keys[i];
			String value = prefs.getString(property);
			setActfConfig(property, value);
		}
	}

	private void setActfConfig(String property, Object value) {
 		String pool = IConfiguration.ACTF_ID;
		int delim = property
				.indexOf(ValidationPreferenceInitializer.POOL_ID_DELIMITER);
		String prefix = delim > 0 ? property.substring(0, delim) : "";
		if (isModelProperty(prefix)) {
			pool = prefix + "_" + IConfiguration.MODEL_ID;
			property = property.substring(delim
					+ ValidationPreferenceInitializer.POOL_ID_DELIMITER.length());
		} else if (prefix.length() > 0
				&& !prefix.equals(IConfiguration.ACTF_ID)) {
			pool = prefix;
			property = property.substring(delim
					+ ValidationPreferenceInitializer.POOL_ID_DELIMITER.length());
		}
		configuration.setSymbolPool(pool);
		if (property.equals(ValidationConstants.REPORTER_LEVEL_KEY)) {
			configuration.setParameter(property, ValidationPreferencePage
					.mapToLevel((String) value));
	} else if (ValidationPreferencePage.isListProperty(property)) {
			configuration.getSymbolPoolContents(property).putAll(
					ValidationPreferenceInitializer.toMap((String) value));
		} else {
			configuration.setParameter(property, value);
		}
	}

	public boolean isModelProperty(String s) {
		String[] arches = configuration.getModelTypes();
		for (int i = 0; i < arches.length; i++) {
			if (arches[i].equals(s)) {
				return true;
			}
		}
		return false;
	}

}
