/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.ui.dialogs;

import java.util.Iterator;
import java.util.Set;

import org.eclipse.actf.validation.ui.GuiUtils;
import org.eclipse.actf.validation.ui.ValidationUIConstants;
import org.eclipse.actf.validation.ui.ValidationUiPlugin;
import org.eclipse.actf.validation.ui.filters.ReportScopeFilter;
import org.eclipse.actf.validation.ui.views.ReportManagerView;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;


public class ReportManagerFilterDialog extends Dialog
{

	protected ReportManagerView view = null;

	// scope radio buttons
	protected Button allScopeButton = null;

	protected Button sourceScopeButton = null;

	protected Combo sourceCombo = null;

	protected Button inactiveButton = null;

	// restore Defaults button
	protected Button restoreDefaultsButton = null;

	// field values
	protected int scope = ReportScopeFilter.SCOPE_ALL;

	protected String toolValue, sourceValue, userValue;

	protected boolean inactive = false;

	public ReportManagerFilterDialog (Shell parent, ReportManagerView dbrv) {
		super(parent);
		view = dbrv;
	}

	protected void configureShell (Shell shell) {
		super.configureShell(shell);
		shell.setText("Report Manager Filters");
	}

	protected Control createDialogArea (Composite parent) {
		initializeDialogUnits(parent);
		Composite area = (Composite) super.createDialogArea(parent);
		GridLayout layout = new GridLayout();
		area.setLayout(layout);
		createScopeGroup(area);
		createInactiveButton(area);
		createDefaultsButton(area);
		loadSettings();
		return area;
	}

	private void createInactiveButton (Composite parent) {
		inactiveButton = new Button(parent, SWT.CHECK);
		GridData gd = new GridData();
		gd.horizontalAlignment = SWT.LEFT;
		inactiveButton.setLayoutData(gd);
		GuiUtils.configureControl(
			inactiveButton, "includeCheck",
			"Select to include inctive entries", "Include inactive reports");
	}

	private void createDefaultsButton (Composite parent) {
		restoreDefaultsButton = new Button(parent, SWT.PUSH);
		GridData gd = new GridData();
		gd.horizontalAlignment = SWT.RIGHT;
		restoreDefaultsButton.setLayoutData(gd);
		GuiUtils.configureControl(
			restoreDefaultsButton, "restoreDefaults",
			"Restore default selections", "Restore Defaults");
		SelectionListener listener = new SelectionAdapter() {

			public void widgetSelected (SelectionEvent e) {
				setDefaults();
			}
		};
		restoreDefaultsButton.addSelectionListener(listener);
	}

	private void createScopeGroup (Composite parent) {
		// scope filters group
		Group scopeGroup = new Group(parent, SWT.NULL);
		scopeGroup.setText("Show reports with the scope of:");
		scopeGroup.setLayout(new FormLayout());
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.widthHint = 400; // to prevent title bar truncation
		scopeGroup.setLayoutData(gd);
		allScopeButton = new Button(scopeGroup, SWT.RADIO);
		GuiUtils.configureControl(
			allScopeButton, "allRadio", "Selects all scopes", "All");
		GuiUtils.configureCommand(allScopeButton, new Runnable() {

			public void run () {
				scope = ReportScopeFilter.SCOPE_ALL;
			}
		});
		
		sourceScopeButton = new Button(scopeGroup, SWT.RADIO);
		GuiUtils.configureControl(
			sourceScopeButton, "sourceRadio", "Selects by source", "By Source");
		GuiUtils.configureCommand(sourceScopeButton, new Runnable() {

			public void run () {
				scope = ReportScopeFilter.SCOPE_SOURCE;
			}
		});
		sourceCombo = new Combo(scopeGroup, SWT.DROP_DOWN);
		GuiUtils.configureControl(
			sourceCombo, "sourceCombo", "Select the desired source");
		GuiUtils.configureCommand(sourceCombo, new Runnable() {

			public void run () {
				sourceValue = sourceCombo.getItem(sourceCombo.getSelectionIndex());
				sourceCombo.setText(sourceValue);
			}
		});
		addSources(sourceCombo); //fill the list box
		// layout the controls in the group box
		GuiUtils.configureLayout(
			allScopeButton, new FormAttachment(0, 5), new FormAttachment(0, 5),
			null, null);
		
		GuiUtils.configureLayout(
			sourceScopeButton, new FormAttachment(0, 5),
			new FormAttachment(allScopeButton, 5), null, null);
		
		GuiUtils.configureLayout(
			sourceCombo, new FormAttachment(sourceScopeButton, 5),
			new FormAttachment(allScopeButton, 5), new FormAttachment(100, -5), null);
	}

	private void setDefaults () {
		// All is the default scope
		scope = ReportScopeFilter.SCOPE_ALL;
		//show inactive reports is false
		inactive = false;
		setControls();
	}

	protected void okPressed () {
		String scopeValue = null;
		inactive = inactiveButton.getSelection();
		scope = getScope();
		if (scope == ReportScopeFilter.SCOPE_SOURCE) {
			scopeValue = sourceValue;
		}
		saveSettings();
		view.setReportFilters(inactive, scope, scopeValue);
		super.okPressed();
	}

	private void saveSettings () {
		IDialogSettings settings = ValidationUiPlugin.getDefault().getDialogSettings();
		try {
			settings.put(ValidationUIConstants.KEY_SHOW_INACTIVE_REPORT, inactive);
			settings.put(ValidationUIConstants.KEY_REPORT_SCOPE, scope);
			settings.put(ValidationUIConstants.KEY_SOURCE_SCOPE_VALUE, sourceValue);
				}catch (Exception e) {
			GuiUtils.reportException("Error saving dialog settings", e);
		}
	}

	private void loadSettings () {
		IDialogSettings settings = ValidationUiPlugin.getDefault().getDialogSettings();
		try {
			inactive = settings.getBoolean(ValidationUIConstants.KEY_SHOW_INACTIVE_REPORT);
			scope = settings.getInt(ValidationUIConstants.KEY_REPORT_SCOPE);
			sourceValue = settings.get(ValidationUIConstants.KEY_SOURCE_SCOPE_VALUE);
			
		}catch (Exception e) { //if any of these come back with no setting, use the default for all
			setDefaults();
		}
		setControls();
	}

	private void setControls () {
		switch (scope) {
		case ReportScopeFilter.SCOPE_ALL:
			allScopeButton.setSelection(true);
			break;
		case ReportScopeFilter.SCOPE_SOURCE:
			sourceScopeButton.setSelection(true);
			break;
		
		}
		if (sourceValue != null) {
			sourceCombo.setText(sourceValue);
		}
		
		inactiveButton.setSelection(inactive);
	}

	
	private void addSources (Combo c) {
		addItems(c, view.querySources());
	}

	private void addItems (Combo c, Set s) {
		if (s != null) {
			Iterator iter = s.iterator();
			while (iter.hasNext()) {
				String item = (String) iter.next();
				addItem(c, item);
			}
		}
		if (c.getItemCount() > 0) {
			c.setText(c.getItem(0));
		}
	}

	private void addItem (Combo c, String s) {
		if (s != null && s.length() > 0) {
			c.add(s);
		}
	}

	private int getScope () {
		int i = ReportScopeFilter.SCOPE_ALL;
		if (allScopeButton.getSelection()) {
			i = ReportScopeFilter.SCOPE_ALL;
		
		}else if (sourceScopeButton.getSelection()) {
			i = ReportScopeFilter.SCOPE_SOURCE;
		}
		return i;
	}
}
