/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*  Kentarou FUKUDA - [232278]
*******************************************************************************/ 

package org.eclipse.actf.validation.ui.jobs;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.actf.validation.ValidationConstants;
import org.eclipse.actf.validation.core.ExecutionPoint;
import org.eclipse.actf.validation.core.ValidationProcessConstants;
import org.eclipse.actf.validation.core.ValidationProcessManager;
import org.eclipse.actf.validation.events.IReportModifyListener;
import org.eclipse.actf.validation.events.ReportModifyEvent;
import org.eclipse.actf.validation.reports.CreationContext;
import org.eclipse.actf.validation.reports.ValidationReport;
import org.eclipse.actf.validation.reports.ValidationReportRecord;
import org.eclipse.actf.validation.ui.ValidationUIConstants;
import org.eclipse.actf.validation.ui.ValidationUiPlugin;
import org.eclipse.actf.validation.ui.preferences.ValidationPreferenceInitializer;
import org.eclipse.actf.validation.ui.preferences.ValidationPreferencePage;
import org.eclipse.actf.validation.ui.views.AbstractViewPart;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;

public abstract class ValidationJob extends Job
{

	public static final QualifiedName NEW_ACTION_QNAME = new QualifiedName(ValidationUiPlugin.getDefault().getPluginId(), "newAction");
	public static final QualifiedName BUNDLE_RESOURCE_QNAME = new QualifiedName(ValidationUiPlugin.getDefault().getPluginId(), "bundleResource");

	protected static ValidationUiPlugin plugin = ValidationUiPlugin.getDefault();
	
	protected ValidationProcessManager vpm;
	protected Map parentMap = new HashMap();
	protected IResource resource;
	protected String resourceName;
	protected ValidationReport currentReport;
	protected Map contextMap = new HashMap();
	protected IValidatableContainer part;
	protected double startTime = 0.0, elapsedTime = 0.0;

	private IReportModifyListener rmListener = new IReportModifyListener() {
		public void reportModified (ReportModifyEvent e) {
			if (e.addedElements instanceof CreationContext[]) {
				for (int i = 0; i < e.addedElements.length; ++i) {
					CreationContext context = (CreationContext) e.addedElements[i];
					plugin.trace(IReporter.CONFIG,
						"reportModify(ReportModifyEvent): Caching " + context);
					contextMap.put(context.getComponentID(), context);
				}
			}
		}
	};

	public ValidationJob (String name) {
		super(name);
		setUser(true);
		setPriority(Job.BUILD);
		setProperty(NEW_ACTION_QNAME, Boolean.TRUE);
	}

	/**
	 * must be called prior to scheduling this job
	 * 
	 * @param obj - used to initialize the job
	 * @throws Exception
	 */
	public abstract void initialize (Object obj) throws Exception;
	
	protected abstract void reset () throws Exception;
	
	public ValidationReport getCurrentReport () {
		return currentReport;
	}

	public IResource getResource () {
		return resource;
	}

	public void setResource (IResource resource) {
		this.resource = resource;
	}

	public void setValidatableContainer (IValidatableContainer part) {
		this.part = part;
	}

	protected IStatus run (IProgressMonitor monitor) {
		final IStatus[] status = new IStatus[] {Status.OK_STATUS};
		
		PlatformUI.getWorkbench().getDisplay().syncExec(new Runnable() {
			public void run () {
				try {
					if (((Boolean) getProperty(NEW_ACTION_QNAME)).booleanValue()) {
						reset();
					}
					preValidation();
					if (part != null) {
						validate();
						postValidation();
					}
				} catch (Throwable e) {
					plugin.logException(e);
					status[0] = new Status(IStatus.ERROR, plugin.getPluginId(), IStatus.ERROR, "Error running validation", e);
				}
			}
		});

		return status[0];
	}

	protected void preValidation () throws Exception {
		IPreferenceStore store = plugin.getPreferenceStore();
		int reportLevel = ValidationPreferencePage.mapToLevel(store.getString(ValidationPreferenceInitializer.P_VALIDATION_LEVEL));
		IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
		
		context.getConfiguration().setParameter(ValidationConstants.REPORTER_LEVEL_KEY, reportLevel);
		startTime = System.currentTimeMillis();
		((IWorkbenchPart) part).setFocus();
	}
	
	protected void validate () throws Exception {
		vpm = ValidationProcessManager.getInstance();
		ExecutionPoint eventExPt = vpm.createMethodExecutionPointForTarget(
			ValidationProcessConstants.VISIBILITY, part.getValidatable(), null, null);
		vpm.initiateValidation(eventExPt, parentMap);
	}
		
	protected void postValidation () throws Exception {
		final ValidationReportRecord[] records = vpm.getReporter().getRecords();
		PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
			public void run () {
				try {
					if (currentReport == null) {
						configureReportInstance();
					}
					currentReport.addRecords(records);
				}catch (RuntimeException e) {e.printStackTrace(System.err);
					plugin.logException(e);
				}
			}
		});
		updateMarkers(records);
		elapsedTime = (System.currentTimeMillis() - startTime) / 1000.0;
		LoggingUtil.println(LoggingUtil.PROCESS_ALL, "Validation finished, elapsed time - " + elapsedTime);
		
		String model = "<unknown>"; //TODO find the GuiArch and set it here or back when the job was created
		String tool = getName();
		String sourceName = "<unknown>"; //fallback 
		if (resource != null) { //for where a resource is associated with the validation
			sourceName = resource.getName();
		}else if (resourceName != null) { //for where there is no resource (Inspection View or URL) 
			sourceName = resourceName;
		}
		
		AbstractViewPart avp = (AbstractViewPart) AbstractViewPart.findView(ValidationUIConstants.REPORT_TREE_VIEW_ID);
		if (avp == null) {
			avp = (AbstractViewPart) AbstractViewPart.activateView(ValidationUIConstants.REPORT_TREE_VIEW_ID);
		}
		if (avp != null) {
			avp.createReport(currentReport, "Report on " + sourceName, sourceName, tool, model);
		} else {
			throw new RuntimeException("Could not find/activate report viewer");
		}
	}

	protected void configureReportInstance () {
		if (currentReport != null) {
			currentReport.removeReportModifyListener(rmListener);
		}
		currentReport = new ValidationReport();
		contextMap.clear();
		currentReport.addReportModifyListener(rmListener);
	}

	protected void updateMarkers (ValidationReportRecord[] records) throws Exception {
		if (resource != null) {
			resource.deleteMarkers(null, true, IResource.DEPTH_ZERO);
			records = getUniqueRecords(records);
			for (int r = 0; r < records.length; ++r) {
				ValidationReportRecord record = records[r];
				String id = record.getNodeId();
				CreationContext context = (CreationContext) contextMap.get(id);
				if (context != null) {
					IMarker marker = resource.createMarker(ValidationUIConstants.VALIDATION_MARKER_ID);
					StackTraceElement[] elems = context.getStackTrace();
					int level = record.getLevel();
					int index = 0;
					for (int t = 0; t < elems.length; ++t) {
						if (resource.getName().indexOf(elems[t].getFileName()) >= 0) {
							index = t;
							break;
						}
					}
					marker.setAttribute(IMarker.MESSAGE, record.getMessage());
					marker.setAttribute(
						IMarker.LOCATION, elems[index].getLineNumber());
					marker.setAttribute(
						IMarker.LINE_NUMBER, elems[index].getLineNumber());
					if (level == IReporter.ERROR) {
						marker.setAttribute(
							IMarker.SEVERITY, IMarker.SEVERITY_ERROR);
					}else if (level == IReporter.WARNING) {
						marker.setAttribute(
							IMarker.SEVERITY, IMarker.SEVERITY_WARNING);
					}else if (level == IReporter.INFO) {
						marker.setAttribute(
							IMarker.SEVERITY, IMarker.SEVERITY_INFO);
					}
				}
			}
		}
	}

	public String getResourceName () {
		return resourceName;
	}

	public void setResourceName (String resourceName) {
		this.resourceName = resourceName;
	}
	public ValidationReportRecord [] getUniqueRecords(ValidationReportRecord[] records) {
		List<ValidationReportRecord> uniqueRecords = new ArrayList<ValidationReportRecord>();
		for (int i = 0; i < records.length; i++) {
			ValidationReportRecord newRecord = records[i];
			Iterator<ValidationReportRecord> recIter = uniqueRecords.listIterator();
			boolean foundMatch = false;
			while (recIter.hasNext()) {
				ValidationReportRecord rec = (ValidationReportRecord) recIter.next();
				if (rec != null && rec.equals(newRecord)) {
					foundMatch = true;
					break;
				}
			}
			if (!foundMatch) {
				uniqueRecords.add(newRecord);
			}
		}
		return (ValidationReportRecord[]) uniqueRecords
		.toArray(new ValidationReportRecord[uniqueRecords.size()]);
	}
}