/*******************************************************************************
* Copyright (c) 2004, 2008 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*  Kentarou FUKUDA - [232278]
*******************************************************************************/ 

package org.eclipse.actf.validation.ui.preferences;


import org.eclipse.actf.validation.ui.ValidationUiPlugin;
import org.eclipse.actf.validation.ui.filters.SeverityFilter;
import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.ColorFieldEditor;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;


/**
 * @author Mike Squillace
 */
public class ValidationPreferencePage extends PreferencePage implements
		IWorkbenchPreferencePage {

	

	private BooleanFieldEditor useDefaultBkgdColorEditor;

	private ColorFieldEditor errorBkgdColorEditor;

	private ColorFieldEditor warnBkgdColorEditor;

	private ColorFieldEditor infoBkgdColorEditor;
	
	private Combo levelCombo;
	private Label levelLabel;

	private int validationLevel;
	
	


	
	public static int mapToLevel (String levelKey) {
		int level = ValidationPreferenceInitializer.WARNINGS;
		if (levelKey.equals(ValidationPreferenceInitializer.V_ALL)) {
			level = ValidationPreferenceInitializer.PROCESS_ALL;
		}else if (levelKey.equals(ValidationPreferenceInitializer.V_INFO)) {
			level = ValidationPreferenceInitializer.PROCESS_INFO;
		}else if (levelKey.equals(ValidationPreferenceInitializer.V_WARNING)) {
			level = ValidationPreferenceInitializer.WARNINGS;
		}else if (levelKey.equals(ValidationPreferenceInitializer.V_ERROR)) {
			level = ValidationPreferenceInitializer.ERRORS;
		}else if (levelKey.equals(ValidationPreferenceInitializer.V_NONE)) {
			level = ValidationPreferenceInitializer.NONE;
		}
		return level;
	}
	
	public static boolean isListProperty (String p) {
		return p.equals(ValidationPreferenceInitializer.P_MODELS)
				|| p.endsWith(ValidationPreferenceInitializer.P_ALIASES);
	}

	
	public ValidationPreferencePage() {
		super();
		setPreferenceStore(ValidationUiPlugin.getDefault().getPreferenceStore());
	}

	public void init(IWorkbench workbench) {
	}

	protected Control createContents(Composite parent) {
		TabFolder folder = new TabFolder(parent, SWT.NONE);
		folder.setLayout(new TabFolderLayout());
		folder.setLayoutData(new GridData(GridData.FILL_BOTH));
		TabItem item = new TabItem(folder, SWT.NONE);
		item.setText("Display");
		item.setControl(createDisplayTab(folder));
		initialize();
		return folder;
	}

	private Control createDisplayTab(Composite parent) {
		GridData gd = null;
		Composite displayComposite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		displayComposite.setLayout(layout);
		gd = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
		displayComposite.setLayoutData(gd);
		useDefaultBkgdColorEditor = new BooleanFieldEditor(
				ValidationPreferenceInitializer.P_COLORS_NORMAL,
				"Use Default Background Color", displayComposite);
		Composite bkgdComposite = new Composite(displayComposite, SWT.NONE);
		GridLayout bkgdLayout = new GridLayout(2, false);
		bkgdComposite.setLayout(bkgdLayout);
		gd = new GridData(GridData.HORIZONTAL_ALIGN_FILL
				| GridData.GRAB_HORIZONTAL);
		gd.horizontalSpan = 3;
		bkgdComposite.setLayoutData(gd);
		errorBkgdColorEditor = new ColorFieldEditor(
				ValidationPreferenceInitializer.P_COLOR_ERROR,
				"Error Background Color:", bkgdComposite);
		errorBkgdColorEditor.getColorSelector().getButton().setToolTipText(
				"Select the error background color");
		warnBkgdColorEditor = new ColorFieldEditor(
				ValidationPreferenceInitializer.P_COLOR_WARNING,
				"Warning Background Color:", bkgdComposite);
		warnBkgdColorEditor.getColorSelector().getButton().setToolTipText(
				"Select the warning background color");
		infoBkgdColorEditor = new ColorFieldEditor(
				ValidationPreferenceInitializer.P_COLOR_INFO, "Info Background Color:",
				bkgdComposite);
		infoBkgdColorEditor.getColorSelector().getButton().setToolTipText(
				"Select the info background color");
		
		// This filed determines the validation level displayed
		levelLabel = new Label(bkgdComposite,SWT.LEFT);
		levelLabel.setText("Validation Level");
		levelCombo = new Combo(bkgdComposite, SWT.READ_ONLY | SWT.DROP_DOWN);
		levelCombo.setItems(SeverityFilter.severityKeys);
		
		return displayComposite;
	}

	private void initialize() {
		initializeDisplay();
	}

	private void initializeDisplay() {
		useDefaultBkgdColorEditor.setPreferenceStore(getPreferenceStore());
		useDefaultBkgdColorEditor.load();
		errorBkgdColorEditor.setPreferenceStore(getPreferenceStore());
		errorBkgdColorEditor.load();
		warnBkgdColorEditor.setPreferenceStore(getPreferenceStore());
		warnBkgdColorEditor.load();
		infoBkgdColorEditor.setPreferenceStore(getPreferenceStore());
		infoBkgdColorEditor.load();
		validationLevel = mapToLevel(getPreferenceStore().getString(ValidationPreferenceInitializer.P_VALIDATION_LEVEL));
		levelCombo.select(validationLevel); 
	}

	protected void performDefaults() {
		
		performDisplayDefaults();
		super.performDefaults();
	}

	

	protected void performDisplayDefaults() {
		useDefaultBkgdColorEditor.loadDefault();
		errorBkgdColorEditor.loadDefault();
		warnBkgdColorEditor.loadDefault();
		infoBkgdColorEditor.loadDefault();
		validationLevel = mapToLevel(getPreferenceStore().getDefaultString(ValidationPreferenceInitializer.P_VALIDATION_LEVEL));
		levelCombo.select(validationLevel);
	}

	public boolean performOk() {
		
		performDisplayOk();
		return super.performOk();
	}

	

	protected void performDisplayOk() {
		useDefaultBkgdColorEditor.store();
		errorBkgdColorEditor.store();
		warnBkgdColorEditor.store();
		infoBkgdColorEditor.store();
		getPreferenceStore().setValue(ValidationPreferenceInitializer.P_VALIDATION_LEVEL, levelCombo.getItem(levelCombo.getSelectionIndex()));
	
	}
}
