/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html

* Contributors: Initial API - Ann Ford
*	
*******************************************************************************/
package org.eclipse.actf.validation.ui.wizards;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.actf.validation.valdoc.rulebase.ReflectionValidationDocument;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.xml.sax.SAXException;

public class RulebaseNameAndTypePage extends WizardPage {
	public static final String PAGE_NAME = "Rulebase Name and Target";

	private Text rulebaseNameText = null;
	private Combo targetTypeCombo = null;
	private boolean nameComplete = false;
	private boolean targetTypeComplete = false;
	private File sourceFile = null;
	private  HashMap targetMap = null;
	

	public RulebaseNameAndTypePage() {
		super(PAGE_NAME);
		setTitle("Rulebase Name and Target Type");
		setDescription("Enter the name of the rulebase and select when to apply it");
		//TODO AKF these are hardcoded for now, should be gotten dynamically
		targetMap = new HashMap(); 
		Set set = new HashSet();
		set.add("org\\.mozilla\\.interfaces\\.nsI.+");
		targetMap.put("Mozilla web", set); //"org\.mozilla\.interfaces\.nsI.+"
		//Swing has 2 so we put them into a set
		//<actf:target actf:regexp="java\.awt\..+"/>
  		//<actf:target actf:regexp="javax\.swing\..+"/>
		set = new HashSet();
		set.add("java\\.awt\\..+");
		set.add("javax\\.swing\\..+");
		targetMap.put("Swing", set); //see above
		set = new HashSet();
		set.add("org\\.eclipse\\.swt\\..+");
		targetMap.put("SWT", set); // "org\.eclipse\.swt\..+"
		
		
	}

	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.NONE);
		composite.setLayoutData(new GridData(GridData.FILL_BOTH));
		GridLayout layout = new GridLayout(2, false);
		composite.setLayout(layout);

		// Row 1 - Rulebase Name

		Label nameLabel = new Label(composite, SWT.LEFT);
		nameLabel.setText("Rulebase Name");
		rulebaseNameText = new Text(composite, SWT.SINGLE);
		rulebaseNameText
				.setToolTipText("Enter the name of the rulebase.  This should match the value of the actf:name attribute of the root <actf:rulebase> element.");
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		rulebaseNameText.setLayoutData(data);
		rulebaseNameText.addModifyListener(new ModifyListener() {

			public void modifyText(ModifyEvent e) {
				if (rulebaseNameText != null) {
					String text = rulebaseNameText.getText();
					if (text != null && text.trim().length() > 0) {
						nameComplete = true;
					} else {
						nameComplete = false;
					}
					getWizard().getContainer().updateButtons();
				}

			}
		});

		// Row 2 - applicable target
		Label targetLabel = new Label(composite, SWT.LEFT);
		targetLabel.setText("Target application type");
		
		targetTypeCombo = new Combo(composite, SWT.DROP_DOWN | SWT.READ_ONLY);
		targetTypeCombo
				.setToolTipText("Select the component type for when to use this rulebase.");
		data = new GridData(GridData.FILL_HORIZONTAL);
		targetTypeCombo.setLayoutData(data);
		Object [] keys = targetMap.keySet().toArray();
		for (int i=0;i<keys.length; i++){
			targetTypeCombo.add((String)keys[i]);
		}	
		targetTypeCombo.addSelectionListener(new SelectionAdapter() {

			public void widgetSelected(SelectionEvent e) {
				int selection = targetTypeCombo.getSelectionIndex();
				targetTypeComplete = true;
				getWizard().getContainer().updateButtons();
			}
		});
		;
		setControl(composite);
	}

	/*
	 * The name of the rulebase should match the value of the actf:name
	 * attribute of the root <actf:rulebase> element. We should first open the
	 * rulebase file and get the value of the 'name; attribute and then use it
	 * to fill in the rulebase name field
	 */
	public String getActualRulebaseName(File sourceFile) {

		String name = null;

		InputStream ios = null;
		if (sourceFile != null) {
			URL url = null;
			try {
				url = sourceFile.toURL();
				if (url != null) {
					try {
						ios = url.openStream();
					} catch (IOException e) {
						setErrorMessage("Unable to open the rulebase file"
								+ sourceFile.getPath());
					}
					if (ios != null) {
						// parse rulebase file for actf:name
						ReflectionValidationDocument valDoc = null;
						try {
							valDoc = new ReflectionValidationDocument(ios);
							ios.close();
						} catch (IOException ioe) {
							setErrorMessage("Unable to open the rulebase file"
									+ sourceFile.getPath());
						} catch (SAXException se) {
							setErrorMessage("Invalid rulebase file"
									+ sourceFile.getPath());
						} catch (Exception e) {
							setErrorMessage("Unknown error when processing the rulebase file"
									+ sourceFile.getPath());
						}

						if (valDoc != null) {
							name = valDoc.getDocumentName();
						}
					}
				}
			} catch (MalformedURLException e) {
				setErrorMessage("Unable to get rulebase file.");
			}

		}
		return name;

	}

	@Override
	public boolean isPageComplete() {
		if (nameComplete && targetTypeComplete && testName()) {
			return saveData();
		} else {
			return false;
		}
	}

	private boolean testName() {
		if (sourceFile != null) {
			String actualName = getActualRulebaseName(sourceFile);
			if (actualName != null
					&& actualName.equals(rulebaseNameText.getText())) {
				setErrorMessage(null);
				return true;
			} else {
				setErrorMessage("The specifed rulebase name does not match the actf:name attribute of the source file - "
						+ actualName);
				return false;
			}

		}
		return false;
	}

	private boolean saveData() {
		LoadRulebaseWizard wizard = (LoadRulebaseWizard) getWizard();
		wizard.setRulebaseName(rulebaseNameText.getText());
		int selection = targetTypeCombo.getSelectionIndex();
		String item = targetTypeCombo.getItem(selection);
		Set regexSet = (Set)targetMap.get(item);
		wizard.setTargetTypeExpression(regexSet); 
		return true;
	}

	public void setRulebaseName(File sourceFile) {
		if (sourceFile != null) {
			this.sourceFile = sourceFile;
			String actualName = getActualRulebaseName(sourceFile);
			if (actualName != null) {
				rulebaseNameText.setText(actualName);
			}
		}
	}

	public void setSourceFile(File sourceFile) {
		this.sourceFile = sourceFile;
	}
}
