/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html

* Contributors: Initial API - Ann Ford
*	
*******************************************************************************/
package org.eclipse.actf.validation.ui.wizards;

import java.io.File;
import java.net.URL;

import org.eclipse.actf.validation.ui.GuiUtils;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

public class TargetLocationPage extends WizardPage {

	public static final String PAGE_NAME = "Target Location";
	public static final String INVALID_DIR = "This is not a valid directory.  Please enter a valid directory path";
	public static final String CANNOT_WRITE_FILE = "Access is denied to this directory";
	public static final String SAME_DIRECTORY_ERROR = "Source location and target location cannot be the same";
	public static final String CANNOT_GET_URL = "Unable to get remote file";

	private Text targetText = null;
	private File saveDir = null;
	private boolean saveLocationComplete = false;

	protected TargetLocationPage() {
		super(PAGE_NAME);
		setTitle("Target Location for new Rulebase");
		setDescription("Specify where to copy the file. This must be on the local file system.");
	}

	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.NONE);
		composite.setLayoutData(new GridData(GridData.FILL_BOTH));
		GridLayout layout = new GridLayout(3, false);
		composite.setLayout(layout);

		Label targetLabel = new Label(composite, SWT.LEFT);
		targetLabel.setText("Target location");
		targetText = new Text(composite, SWT.SINGLE);
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		targetText.setLayoutData(data);
		targetText
				.setToolTipText("Enter the directory location to save the file or use the Browse button to select one");
		targetText.addModifyListener(new ModifyListener() {

			public void modifyText(ModifyEvent e) {
				if (targetText != null) {
					String text = targetText.getText();
					if (text != null && text.trim().length() > 0) {
						saveLocationComplete = true;		
					} else {
						saveLocationComplete = false;
					}
					getWizard().getContainer().updateButtons();
				}

			}

		});
		Button targetButton = new Button(composite, SWT.PUSH);
		GuiUtils.configureControl(targetButton, "targetLocation",
				"Choose a location to save the file", "Browse...");
		SelectionListener targetListener = new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				saveDir = getSaveDirectory();
				targetText.setText(saveDir.getPath());
			}
		};
		targetButton.addSelectionListener(targetListener);

		setControl(composite);
	}

	private File getSaveDirectory() {
		DirectoryDialog fd = new DirectoryDialog(getShell(), SWT.SAVE);
		File dir = null;
		String path = fd.open();
		if (path != null && path.length() > 0) {
			dir = new File(path);
		}
		return dir;

	}

	@Override
	public boolean canFlipToNextPage() {
		return isPageComplete();
	}

	@Override
	public IWizardPage getNextPage() {
		LoadRulebaseWizard wizard = (LoadRulebaseWizard) getWizard();
		RulebaseNameAndTypePage page = (RulebaseNameAndTypePage) wizard
				.getPage(RulebaseNameAndTypePage.PAGE_NAME);
		File sourceFile = null;
		if (wizard.isFileLocal()) {
			sourceFile = wizard.getSourceFile();
		} else {
			
				sourceFile = wizard.getRemoteRulebaseFile();
			
			if (sourceFile == null) {
				setErrorMessage(CANNOT_GET_URL);
			}
		}

		if (page != null && sourceFile != null) {
			page.setRulebaseName(sourceFile);
		}
		return page;
	}

	private boolean saveData() {
		LoadRulebaseWizard wizard = (LoadRulebaseWizard) getWizard();

		saveDir = new File(targetText.getText());
		
		if (saveDir != null && !saveDir.isDirectory())  {
			setErrorMessage(INVALID_DIR);
			return false;
		}
		if (!saveDir.canWrite()){
			setErrorMessage(CANNOT_WRITE_FILE);
			return false;
		}

		if (wizard.isFileLocal()) {
			String sourcePath = wizard.getSourceFile().getParent();
			if (sourcePath.equals(saveDir.getPath())) {
				setErrorMessage(SAME_DIRECTORY_ERROR);
				return false;
			}
		} 
		wizard.setSaveLocation(saveDir);
		setErrorMessage(null);
		return true;
	}

	@Override
	public boolean isPageComplete() {
		if (saveLocationComplete) {
			return saveData();
		} else {
			return false;
		}

	}

}
