/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.valdoc.codeproc;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import org.eclipse.actf.core.config.ConfigurationException;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.model.IModel;

/**
 * default implementation of a factory
 * 
 * @author Mike Squillace
 */
public class DefaultCodeProcessorFactory implements ICodeProcessorFactory {

	/**
	 * create a new DefaultCodeProcessorFactory and use the .properties file
	 * packaged with ACTF to find processor
	 * 
	 */
	public DefaultCodeProcessorFactory() {
		this((InputStream) null);
	}

	/**
	 * create a new DefaultCodeProcessorFactory and use the .properties file
	 * with the given name to find processor
	 * 
	 * @param filename --
	 *            nname of .properties file to use
	 * @throws IOException
	 */
	public DefaultCodeProcessorFactory(String filename) throws IOException {
		this(new FileInputStream(filename));
	}

	/**
	 * create a new DefaultCodeProcessorFactory and use the specified stream to
	 * find processor
	 * 
	 * @param stream --
	 *            stream for resolving language to ICodeProcessor
	 */
	public DefaultCodeProcessorFactory(InputStream stream) {
	}

	/**
	 * retrieve a ICodeProcessor for the given language. The IBM Reflexive GUI
	 * Builder currently supports the following scripting languages:
	 * 
	 * <p>
	 * <ul>
	 * <li>JavaScript/Rhino
	 * </ul>
	 * 
	 *  
	 * <p>
	 * Note: If the supplied model is <code>null</code>, a processor
	 * will still be generated but the IGuiModel implementation cannot be
	 * used within the context of construction or validation of GUIs.
	 * 
	 * @param lang --
	 *            language for which a AbstractCodeInterpreter is to be
	 *            instantiated
	 * @param model --
	 *            model for which code interpreter is to be built (may be
	 *            <code>null</code>)
	 * @return code processor
	 * @see "resources/actf.xml"
	 */
	public ICodeProcessor getCodeProcessor(String lang, IModel model) {
		IConfiguration config = null;
		IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
		try {
			config = context.getConfiguration();
		} catch (ConfigurationException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
		
		if (model == null) {
			throw new RuntimeException("IModel cannot be null");
		}
		ICodeProcessor ci = null;
		try {
			if (config != null) {
				config.setSymbolPool(ICodeProcessor.SCRIPTLANG_ID);
				if (lang != null) {
					ci = (ICodeProcessor) config.getClassParameter(lang).getConstructor(
									new Class[] { String.class, IModel.class })
										.newInstance(new Object[] { lang, model });
				}
			}
		} catch (Throwable e) {
			throw new IllegalArgumentException("Cannot form code processor for specified language: " + lang, e);
		}
		
		return ci;
	} // getCodeProcessor
} // DefaultCodeProcessorFactory
