/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.valdoc.rulebase;

import org.eclipse.actf.validation.core.ExecutionPointDescriptor;
import org.eclipse.actf.validation.rulebase.MalformedRuleBaseException;
import org.w3c.dom.Element;


/**
 * represents a validation document for document rulebases
 * 
 * @author <a href="mailto:masquill@us.ibm.com">Mike Squillace</a>
 *
 */
public interface IValidationDocument
{

	// rulebase attributes
	
	/** attribute for document enablement; value = 'enable' */
	public static final String ENABLE = "enable";
	
	/** attribute for model; value = 'model' */
	public static final String MODEL = "model";

	/** attribute for document name; value = 'name' */
	public static final String DOCUMENT_NAME = "name";

	/** attribute for document script language (if any); value = 'scriptlang' */
	public static final String SCRIPTLANG = "scriptlang";

	/** attribute for document version; value = 'version' */
	public static final String DOCUMENT_VERSION = "version";
	
	/** namespace URI for all ACTF documents; value is 'http://www.eclipse.org/projects/actf' */
	public static final String ACTF_NAMESPACE_URI = "http://www.eclipse.org/projects/actf/validation";

	/**
	 * return the value of the actf:name attribute for this document
	 * 
	 * @return name of document or <code>null</code> if no name is specified
	 */
	public String getDocumentName();

	/**
	 * return the value of the actf:model attribute
	 * 
	 * @return value of actf:enable attribute or <code>null</code> if
	 *         not present
	 */
	public String getModelType();

	/**
	 * return the value of the actf:enable attribute
	 * 
	 * @return value of actf:enable attribute or <code>null</code> if
	 *         not present
	 */
	public String getEnable();

	/**
	 * return the value of the actf:scriptlang attribute
	 * 
	 * @return value of actf:scriptlang attribute or <code>null</code> if not
	 *         present
	 */
	public String getScriptLanguage();

	/**
	 * return an array of script elements of the document. These elements
	 * include immediate children of the &lt;raven:scripts&gt; element. If the
	 * &lt;raven:scripts&gt; has only text content, then a single script element
	 * should be returned.
	 * 
	 * @return array of script elements or an empty array if no script elements
	 *         are present
	 */
	public ScriptElement[] getScriptElements() throws MalformedRuleBaseException;

	/**
	 * return an array of alias elements of the document. These elements include
	 * immediate children of the &lt;raven:aliases&gt; element and must have a tag name of 'raven:alias'.
	 * 
	 * @return array of alias elements or an empty array if no alias elements are present
	 */
	public AliasElement[] getAliasElements() throws MalformedRuleBaseException;

	/**
	 * return the list of ruleset elements of the document. These elements include
	 * immediate children of the &lt;raven:rulesets&gt; element and must have a tag name of 'raven:ruleset'.
	 * 
	 * @return array of ruleset elements or an empty array if no ruleset elements are present
	 */
	public Element[] getRuleSetElements();

	/**
	 * retreave the set of execution point descriptors stipulated in this document. Execution points are used to 
	 * determine when processes are to be started during the executing application or rendering of  
	 * or interaction with a document.
	 * 
	 * @return execution point descriptors specified by this document
	 */
	public ExecutionPointDescriptor[] getExecutionPointDescriptors ();

	/**
	 * return the value of the actf:version attribute for this document
	 * 
	 * @return version of the document or <code>null</code> if no version is
	 *         specified
	 */
	public String getDocumentVersion();
	
} // IValidationDocument
