/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.valdoc.scripting;

import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

import org.apache.bsf.BSFException;
import org.apache.bsf.BSFManager;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.model.IModel;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.actf.validation.ValidationConstants;
import org.eclipse.actf.validation.valdoc.codeproc.AbstractCodeProcessor;
import org.eclipse.actf.validation.valdoc.codeproc.EvaluationException;
import org.mozilla.javascript.Context;

// import org.eclipse.actf.util.thread.ThreadUtils; // thread debugging import
/**
 * used to evaluate expressions and execute scripts given in the XML document.
 * By default, all text content of elements and expressions within curly braces
 * {...} will be evaluated by the interpreter.
 * 
 * <p>
 * Note: Although this class provides the full implementation of the
 * <code>CodeInterpreter</code> interface, it is recommended that the
 * AbstractCodeInterpreter class be extended by those wishing to use scripting
 * languages not currently supported by or packaged with this application.
 * 
 * @author Mike Squillace
 */
public abstract class AbstractCodeInterpreter extends AbstractCodeProcessor
{

	protected BSFManager manager;
	
	private Map codeMap = new HashMap();
	private boolean languageSupportsTuples = false;

	/**
	 * create a code interpreter for processing scripts and expressions in the
	 * specified language. The specified language must match one of those given
	 * in the raven.xml file.
	 * 
	 * <p>
	 * Note: this constructor calls <code>initClasspath()</code> if a
	 * scripting engine for the specified language can be found.
	 * 
	 * @param language --
	 *            language of scripts in the document to be processed
	 * @param model --
	 *            model to be used
	 * @see "resources/actf.xml"
	 */
	public AbstractCodeInterpreter(String language, IModel model) {
		super(language, model);
		BSFManager.registerScriptingEngine("javascript",
				"org.apache.bsf.engines.javascript.JavaScriptEngine", new String[] { "javascript" });

		manager = new BSFManager();
		try {
			preInit();
			configuration.setSymbolPool(IConfiguration.ACTF_ID);
			setClasspath(configuration.getStringParameter(ValidationConstants.CODE_PROCESSOR_CLASSPATH_KEY));
			manager.loadScriptingEngine(language);
			postInit();
		} catch (Exception e) {
			throw new IllegalArgumentException(e.getMessage());
		}
	} // main constructor

	/**
	 * perform initialization prior to instantiating the underlying script
	 * interpreter. This default implementation does nothing.
	 * 
	 */
	protected void preInit() throws Exception {
	}

	/**
	 * perform initialization after instantiating underlying interpreter. This
	 * default implementation does nothing.
	 * 
	 */
	protected void postInit() throws Exception {
	}

	/** {@inheritDoc} */
	public void setClasspath(String cp) {
		super.setClasspath(cp);
		initClasspath();
	}

	/**
	 * set the environment for this interpreter. This method is primarily to be
	 * called when a new interpreter is instantiated and is to have the same
	 * environment (e.g. local variables) as another interpreter; thus, it
	 * should be called in concert with <code>getEnvironment()</code>
	 * 
	 * @param env --
	 *            environment of another interpreter retreaved via
	 *            <code>getEnvironment()</code>
	 */
	public void setEnvironment(Object env) {
		manager = (BSFManager) env;
	}

	/**
	 * used to retreave the environment of this interpreter so as to be passed
	 * to the <code>setEnvironment(Object)</code> of a newly instantiated
	 * interpreter
	 * 
	 * @return environment of this interpreter
	 * @see #setEnvironment(Object)
	 */
	public Object getEnvironment() {
		return manager;
	}

	/**
	 * initializes the underlying scripting engine with the classpath supplied
	 * via <code>setClasspath</code>. This will be the classpath referenced
	 * from within actual script code.
	 * 
	 * @see #setClasspath(String)
	 */
	protected void initClasspath() {
	}

	/**
	 * return whether or not the language of this interpreter supports "tuples".
	 * Special processing is available for scripting languages that support
	 * "tuples", delimited expressions that can be treated as and evaluated as a
	 * single expression
	 * 
	 * @return <code>true</code> if language supports primitive tuple data
	 *         type, <code>false</code> otherwise
	 */
	public boolean getLanguageSupportsTuples() {
		return languageSupportsTuples;
	}

	/**
	 * the expression delimiter is always a semicolon (;) unless
	 * <code>getLanguageSupportsTuples()</code> returns <code>true</code>
	 * 
	 * @return expression delimiter
	 */
	public char getExpressionDelimiter() {
		return ';';
	}

	/**
	 * set whether or not the scripting language supports tuples
	 * 
	 * @param b --
	 *            <code>true</code> if language does, in fact, support tuples
	 */
	protected void setLanguageSupportsTuples(boolean b) {
		languageSupportsTuples = b;
	}

	/**
	 * execute the script code specified. The interpreter wil, upon execution,
	 * reflect the new environment engendered by this code
	 * 
	 * @param code --
	 *            code to be executed
	 */
	public void addScript(String code) {
		addScript(null, code);
	}

	/**
	 * add the script code with the specified script name to this interpreter
	 * for later execution
	 * 
	 * @param scriptName --
	 *            name or id of script
	 * @param scriptCode --
	 *            code to be executed
	 */
	public void addScript(String scriptName, String scriptCode) {
		LoggingUtil.println(LoggingUtil.ALL,
				"Entering .addScript("+scriptName+", scriptCode)");
		if (scriptCode != null && scriptCode.trim().length() > 0) {
			String preparedCode = prepareCode(scriptCode);
			if (scriptName != null) {
				LoggingUtil.println(LoggingUtil.ALL, "Adding to codeMap.");
				codeMap.put(scriptName, preparedCode);
			} else {
				codeMap.put("__temp_script__", preparedCode);
				LoggingUtil.println(LoggingUtil.ALL,
						"Calling processCode(__temp_script__)");
				processCode(new String[] { "__temp_script__" });
				LoggingUtil.println(LoggingUtil.ALL,
						"After processCode(__temp_script__)");
				codeMap.remove("__temp_script__");
			}
		}
		LoggingUtil.println(LoggingUtil.ALL,
				"Exiting .addScript(scriptName, scriptCode)");
	}

	/**
	 * get the code associated with the given script name
	 * 
	 * @param scriptName --
	 *            name of script for which code is desired
	 * @return code of script with given name or <code>null</code> if name is
	 *         not valid
	 */
	public String getScript(String scriptName) {
		return (String) codeMap.get(scriptName);
	}

	/**
	 * get the initialization code for this interpreter
	 * 
	 * @return initialization code for this interpreter (run once at
	 *         instantiation)
	 */
	public String getInitCode() {
		StringBuffer sb = new StringBuffer();
		String[] packageList = getModel().getPackageNames();

		for (int pl = 0; pl < packageList.length; ++pl) {
			sb.append(formatPackageImport(packageList[pl]));
		}

		return sb.toString();
	}
	
	protected abstract String formatPackageImport (String packageName);
	

	/**
	 * declare a new identifier to this interpreter for use in scripts
	 * 
	 * @param name --
	 *            identifier for this object
	 * @param obj --
	 *            object being declared (this may not be <code>null</code>
	 * @param vParam --
	 *            visibility parameter
	 */
	public void setLocal(String name, Object obj, String vParam) {
		if (Context.getCurrentContext() == null) {
			Context.enter();
		}
		if (isIdVisible(name, vParam) && obj != null) {
			try {
				manager.declareBean(name, obj, obj.getClass());
			} catch (BSFException e) {
				LoggingUtil.println(IReporter.SYSTEM_NONFATAL,
						"In declaring symbol " + name, e);
			}
		} else if (obj == null) {
			addScript("  " + name + "=" + getNullValue());
		}
	} // setLocal

	/**
	 * execute the code in params[0]. Remaining parameters are not currently
	 * supported.
	 * 
	 * @param params --
	 *            name of script to be executed in [0] (the remainder of the
	 *            array is ignored)
	 */
	public boolean processCode(String[] params) {
		String scriptName = params[0];
		String code = (String) codeMap.get(scriptName);
		boolean result = false;
		if (code != null) {
			LoggingUtil.println(LoggingUtil.ALL, "Executing script name - "
					+ scriptName + "script code: \n" + code);
			LoggingUtil.println(LoggingUtil.ALL,
					"Processing code using manager class - "
							+ manager.getClass().getName());
			// ThreadUtils.dumpThreadState("AbstractCodeInterpreter.processCode());
			try {
				manager.exec(getLanguage(), "<STRING>", -1, -1, code);
				result = true;
				LoggingUtil.println(LoggingUtil.ALL,
						"manager.exec() returned successfully.");
			} catch (Exception e) {
				LoggingUtil.println(LoggingUtil.ALL,
						"Exception thrown from manager.exec().");
				if (LoggingUtil.getTraceLevel() <= LoggingUtil.ERRORS) {
					int line = 0;
					StringTokenizer st = new StringTokenizer(code, "\n");
					StringBuffer sb = new StringBuffer();
					sb.append("Interpreter Exception while executing:");
					sb.append('\n');
					while (st.hasMoreTokens()) {
						sb.append(line++ + ":  " + st.nextToken());
						sb.append('\n');
					}
					sb.append('\n');
					LoggingUtil.println(IReporter.SYSTEM_NONFATAL, sb.toString(),
							e);
				}
			}
		} else if (!scriptName.equals("init")) {
			LoggingUtil.println(IReporter.USER, "No script with name "
					+ scriptName);
		}
		return result;
	} // processCode

	/**
	 * attempt to evaluate the given string as an argument of the specified
	 * type. The given string is evaluated according to the syntax and semantics
	 * of the particular interpreter being used.
	 * 
	 * @param code --
	 *            argument to be evaluated
	 * @param returnType --
	 *            desired type of returned object
	 * @return newly created object resulting from evaluation
	 * @throws EvaluationException
	 *             if evaluation was not successful
	 */
	public Object evaluateArguments(String code, Class returnType)
			throws EvaluationException {
		Object[] params = evaluateArguments(code, new Class[] { returnType });
		return (params != null ? params[0] : null);
	}

	/**
	 * attempt to evaluate the given code and return an array of objects
	 * corresponding to the specified types. In the context of a code
	 * interpreter, this method is typically used when
	 * <code>languageSupportsTuples()</code> returns <code>true</code>
	 * 
	 * @param code --
	 *            code to be evaluated
	 * @param paramTypes --
	 *            desired types of returned objects
	 * @return newly created objects resulting from evaluation
	 * @throws EvaluationException
	 *             if evaluation was not successful
	 * @see #getLanguageSupportsTuples()
	 */
	public Object[] evaluateArguments(String code, Class[] paramTypes)
			throws EvaluationException {
		String[] args = makeExpressionList(code);
		Object object = null;
		Object[] params = new Object[paramTypes.length];
		int index = 0;
		try {
			for (int a = 0; a < args.length; ++a) {
				String arg = args[a];
				LoggingUtil.println(LoggingUtil.ALL, "Evaluating " + arg);
				object = manager.eval(getLanguage(), "<STRING>", -1, -1, arg);
				LoggingUtil.println(LoggingUtil.ALL, "Returned with result: "
						+ object.getClass().getName());
				// conversion from special type (e.g. PyObject returned by
				if (getLanguageSupportsTuples() && args.length == 1) {
					try {
						params = toJavaObjects(object, paramTypes);
					} catch (IllegalArgumentException e1) {
						params = null;
						break;
					}
				} else {
					// be sure that there are enough actual parameters remaining
					// to fill in the formal parameters
					if (args.length == paramTypes.length) {
						try {
							params[index] = toJavaObject(object,
									paramTypes[index]);
							LoggingUtil.println(LoggingUtil.ALL,
									"Evaluated parameter "
											+ index
											+ " with result class "
											+ params[index].getClass()
													.getName());
							++index;
						} catch (IllegalArgumentException e1) {
							params = null;
							break;
						}
					} else {
						params = null;
						break;
					}
				}
			} // next arg
		} catch (Throwable e) {
			throw new EvaluationException(
					"Interpreter exception while evaluating: " + code);
		}
		return params;
	} // evaluateArguments

	/**
	 * @return <code>false</code>
	 */
	public boolean isCompilerProcessor() {
		return false;
	}

	/**
	 * overwritten by those interpreters that wrap the results of evaluations in
	 * API-specific instances
	 * 
	 * @param o --
	 *            wrapped object
	 * @param c --
	 *            desired type
	 * @return "unwrapped" o as type c
	 */
	protected abstract Object toJavaObject(Object o, Class c);

	/**
	 * convert the given object to an array of objects of the specified types.
	 * This version is typically used by those interpreters in which
	 * <code>languageSupportsTuples()</code> returns <code>true</code>
	 * 
	 * @param o --
	 *            API-dependent tuple object
	 * @param paramTypes --
	 *            desired types of returned objects
	 * @return array of objects of the specified types or <code>null</code> if
	 *         conversion could not be performed
	 * @see #getLanguageSupportsTuples()
	 */
	protected abstract Object[] toJavaObjects(Object o, Class[] paramTypes);

	/**
	 * return the equivalent of a <code>null</code> value
	 * 
	 * @return null-value equivalent
	 */
	protected String getNullValue() {
		return "null";
	}
} // AbstractCodeInterpreter
