/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  John Buslawski - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.validators;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.LoggingUtil;
import org.eclipse.actf.validation.core.ValidationProcessConstants;

/**
 * The IValidationContext factory.
 * 
 * <p>
 * <b>Note</b>: All process-validator mappings are currently hard-coded in this
 * implementation. Future work will include allowing these mappings to be
 * specified in a rulebase.
 * 
 * @author John Buslawski
 * 
 */
public class DefaultValidationContextFactory implements
		IValidationContextFactory {

	protected static final Map namesToContexts = new HashMap();

	protected static final Map processToContexts = new HashMap();
	static {
		namesToContexts
				.put(ValidationProcessConstants.ACCESSIBILITY,
						"org.eclipse.actf.validation.validators.AccessibilityValidationContext");
		namesToContexts
				.put(ValidationProcessConstants.FOCUS,
						"org.eclipse.actf.validation.validators.FocusValidationContext");
		processToContexts
				.put(
						"org.eclipse.actf.validation.core.AccessibilityValidationProcess",
						"org.eclipse.actf.validation.validators.AccessibilityValidationContext");
		processToContexts
				.put("org.eclipse.actf.validation.core.FocusValidationProcess",
						"org.eclipse.actf.validation.validators.FocusValidationContext");
	}

	// a handle to the singleton ReflectionDocumentCriterionFactory
	static private DefaultValidationContextFactory instance = null;

	/**
	 * @return The unique instance of this class.
	 */
	static public DefaultValidationContextFactory getInstance() {
		if (null == instance) {
			instance = new DefaultValidationContextFactory();
		}
		return instance;
	}

	// default constructor
	protected DefaultValidationContextFactory() {
	}

	/** {@inheritDoc} */
	public void registerContextClassName(String name, String contextClassName) {
		if (name == null || name.length() == 0) {
			return;
		}
		if (contextClassName != null) {
			if (!namesToContexts.containsKey(name)) {
				namesToContexts.put(name, contextClassName);
			}
		}
	}

	/** {@inheritDoc} */
	public void registerContextByProcessName(String processName,
			String contextClassName) {
		if (processName == null || processName.length() == 0) {
			return;
		}
		if (contextClassName != null) {
			if (!processToContexts.containsKey(processName)) {
				processToContexts.put(processName, contextClassName);
			}
		}
	}

	/** {@inheritDoc} */
	public IValidationContext getContextInstance(String name) {
		IValidationContext valCtx = null;
		if (name != null && name.length() > 0) {
			String ctxName = (String) namesToContexts.get(name);
			if (ctxName != null && ctxName.length() > 0) {
				try {
					LoggingUtil
							.println(
									LoggingUtil.PROCESS_ALL,
									"IValidationContextFactory.getContextInstance(String), Creating validation context: "
											+ ctxName);
					valCtx = (IValidationContext) Class.forName(ctxName)
							.newInstance();
				} catch (Exception e) {
					LoggingUtil
							.println(
									IReporter.SYSTEM_NONFATAL,
									"IValidationContextFactory.getContextInstance(String), unable to create validation context: "
											+ ctxName, e);
					throw new IllegalArgumentException(
							"Context type "
									+ ctxName
									+ " is not registered. Unable to create IValidationContext.");
				}
			}
		}
		return valCtx;
	}

	/** {@inheritDoc} */
	public IValidationContext getContextByProcessName(String procName) {
		IValidationContext valCtx = null;
		if (procName != null && procName.length() > 0) {
			String ctxName = (String) processToContexts.get(procName);
			if (ctxName != null && ctxName.length() > 0) {
				try {
					LoggingUtil
							.println(
									LoggingUtil.PROCESS_ALL,
									"IValidationContextFactory.getContextInstance(String), Creating validation context: "
											+ ctxName);
					valCtx = (IValidationContext) Class.forName(ctxName)
							.newInstance();
				} catch (Exception e) {
					LoggingUtil
							.println(
									IReporter.SYSTEM_NONFATAL,
									"IValidationContextFactory.getContextInstance(String), unable to create validation context: "
											+ ctxName, e);
					throw new IllegalArgumentException(
							"Context type "
									+ ctxName
									+ " is not registered. Unable to create IValidationContext.");
				}
			}
		}
		return valCtx;
	}
}
